
// (c) COPYRIGHT URI/MIT 1994-1996
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      reza            Reza Nekovei (reza@intcomm.net)

// DODS-netCDF surrogate library functions for data attributes. See note
// about the symbol LOCAL in Dfile.c
// 
// ReZa 10/4/94

#include "config_nc.h"

static char rcsid[] not_used ={"$Id: Dattr.cc,v 1.5.4.1 2004/06/15 18:43:39 jimg Exp $"};

#include <assert.h>
#include  "Dnetcdf.h"
#include "Dncx.h"

#ifdef __GNUG__
#pragma implementation
#endif

Pix
get_das_table(int cdfid, int varid)
{
  Pix q;

  assert(conns); 

  q = (*conns)[cdfid]->das().first_var();
  for (int i=0; i < (*conns)[cdfid]->das_loc(varid+1) ; i++)
    (*conns)[cdfid]->das().next_var(q);
 
  return q;
}

void
correct_att_type(int cdfid, Pix q, const char *name, void *values)
{
    unsigned int i;
    nclong *lp;
    unsigned long *ulp;
    short *sp;
    unsigned short *usp;
    float *fp;
    double *dp;
    char *cp;

    assert(conns); 

    // from the attribute table get the requested atrribute type by name

    string attrT = (*conns)[cdfid]->das().get_table(q)->get_type(name);

    // from the attribute table get the requested atrribute length by name
    unsigned int attr_siz = (*conns)[cdfid]->das().get_table(q)->get_attr_num(name);

    // integers are converted to the binary and added to the pointer
    if (attrT == "Int16") {
	sp = (short *) values;
	for (i=0; i<attr_siz; i++)
	    *(sp+i) = atol((*conns)[cdfid]->das().get_table(q)->get_attr(name,i).c_str());
    }
    if (attrT == "Int32") {
	lp = (nclong *) values;
	for (i=0; i<attr_siz; i++)
	    *(lp+i) = atol((*conns)[cdfid]->das().get_table(q)->get_attr(name,i).c_str());
    }

    if (attrT == "UInt16" || attrT == "Uint16") {
	usp = (unsigned short *) values;
	for (i=0; i<attr_siz; i++)
	    *(usp+i) = atol((*conns)[cdfid]->das().get_table(q)->get_attr(name,i).c_str());
    }
    if (attrT == "UInt32" || attrT == "Uint32") {
	ulp = (unsigned long *) values;
	for (i=0; i<attr_siz; i++)
	    *(ulp+i) = strtoul((*conns)[cdfid]->das().get_table(q)->get_attr(name,i).c_str(), 0, 0);
    }

    // floats are converted to the binary and added to the pointer address
    if (attrT == "Float32"){
	fp = (float *) values;
	for (i=0; i<attr_siz; i++)
	    *(fp+i)=atof((*conns)[cdfid]->das().get_table(q)->get_attr(name,i).c_str());
    }
    if (attrT == "Float64"){
	dp = (double *) values;
	for (i=0; i<attr_siz; i++)
	    *(dp+i)=atof((*conns)[cdfid]->das().get_table(q)->get_attr(name,i).c_str());
    }

    // bytes are added one at the time to the pointer address
    if (attrT == "Byte"){
	cp = (char *) values;
	for (i=0; i<attr_siz; i++)
	    *(cp+i)=(char) atoi((*conns)[cdfid]->das().get_table(q)->get_attr(name,i).c_str());
    }

    // strings are added to the pointer (seperated by '\n' and ended by '\0')
    if (attrT == "String"){
	cp = (char *) values;
	int offset=0;
	for (i=0; i<attr_siz; i++){
	    string attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name,i);
	    size_t atsiz = attrV.length() - 2 ; // size minus quotes  
	    string Attribute = attrV.substr(1, atsiz); // remove quotes
	    (void) memcpy(cp+offset, Attribute.c_str(), atsiz); 
	    offset += atsiz; 
	    if(attr_siz > 1){
	      *(cp+offset) = '\n'; // Separate strings by EOL for more than 1
	      offset++;            // string (not a DODS netcdf server).
	    }
	}
	if(attr_siz > 1)
	*(cp+offset-1) = '\0';	// End the string attribute by EOS, if array
				// of string was sent (not a netCDF server).
    }

}

int
nc_put_att_text(int cdfid, int varid, const char *name,
        size_t nelems, const char *value)
{
    assert(conns); 

    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_put_att_text((*conns)[cdfid]->ncid(), varid, name, 
			  nelems, value);
	return rcode;
    }
#endif

    return NC_EPERM; // DODS network interface is read-only
}

int
nc_put_att_schar(int cdfid, int varid, const char *name,
        nc_type type, size_t nelems, const signed char *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_put_att_schar((*conns)[cdfid]->ncid(), varid, name, 
			  type, nelems, value);
	return rcode;
    }
#endif

    return NC_EPERM; // DODS network interface is read-only
}

int
nc_put_att_uchar(int cdfid, int varid, const char *name,
        nc_type type, size_t nelems, const unsigned char *value)
{ 
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_put_att_uchar((*conns)[cdfid]->ncid(), varid, name, 
			  type, nelems, value);
	return rcode;
    }
#endif

    return NC_EPERM; // DODS network interface is read-only
}
 
int
nc_put_att_short(int cdfid, int varid, const char *name,
        nc_type type, size_t nelems, const short *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_put_att_short((*conns)[cdfid]->ncid(), varid, name, 
			  type, nelems, value);
	return rcode;
    }
#endif

    return NC_EPERM; // DODS network interface is read-only
}

int
nc_put_att_int(int cdfid, int varid, const char *name,
        nc_type type, size_t nelems, const int *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_put_att_int((*conns)[cdfid]->ncid(), varid, name, 
			  type, nelems, value);
	return rcode;
    }
#endif

    return NC_EPERM; // DODS network interface is read-only
}

int
nc_put_att_long(int cdfid, int varid, const char *name,
        nc_type type, size_t nelems, const long *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_put_att_long((*conns)[cdfid]->ncid(), varid, name, 
			  type, nelems, value);
	return rcode;
    }
#endif

    return NC_EPERM; // DODS network interface is read-only
}


int
nc_put_att_float(int cdfid, int varid, const char *name,
        nc_type type, size_t nelems, const float *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_put_att_float((*conns)[cdfid]->ncid(), varid, name, 
			  type, nelems, value);
	return rcode;
    }
#endif

    return NC_EPERM; // DODS network interface is read-only
}

int
nc_put_att_double(int cdfid, int varid, const char *name,
        nc_type type, size_t nelems, const double *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_put_att_double((*conns)[cdfid]->ncid(), varid, name, 
			  type, nelems, value);
	return rcode;
    }
#endif

    return NC_EPERM; // DODS network interface is read-only
}


int
nc_put_att(int cdfid, int varid, const char *name, nc_type type,
        size_t nelems, const void *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_put_att((*conns)[cdfid]->ncid(), varid, name, 
			  type, nelems, value);
	return rcode;
    }
#endif

    return NC_EPERM; // DODS network interface is read-only
} 


int
ncattput(int cdfid, int varid, const char *name,  nc_type datatype, 
	 int count, const void *values)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncattput((*conns)[cdfid]->ncid(), varid, name, 
			  datatype, count, values);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattput" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}

int
nc_rename_att(int cdfid, int varid, const char *name, const char *newname)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_rename_att((*conns)[cdfid]->ncid(), varid, name, newname);
	return rcode;
    }
#endif

    return NC_EPERM; // DODS network interface is read-only
}

int
ncattrename(int cdfid, int varid, const char *name, const char *newname)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncattrename((*conns)[cdfid]->ncid(), varid, name, newname);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattrename" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}

int
nc_del_att(int cdfid, int varid, const char *name)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_del_att((*conns)[cdfid]->ncid(), varid, name);
	return rcode;
    }
#endif

    return NC_EPERM; // DODS network interface is read-only
}

int
ncattdel(int cdfid, int varid, const char *name)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncattdel((*conns)[cdfid]->ncid(), varid, name);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattdel" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}

int
nc_inq_attname(int cdfid, int varid, int attnum, char *name)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_inq_attname((*conns)[cdfid]->ncid(), varid, attnum, name);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if(( varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) 
      return NC_ENOTVAR;

    if(attnum < 0)
      return  NC_ENOTATT;

    Pix q = get_das_table(cdfid,varid); // get das table

    // from the attribute table get the requested atrribute name
    Pix p = (*conns)[cdfid]->das().get_table(q)->first_attr();
    for (int i=0; (i < attnum) && p ; i++)
	(*conns)[cdfid]->das().get_table(q)->next_attr(p);

    if(p == (Pix)NULL) // attnum is greater than the attributes in the table
      return NC_ENOTATT;
    else
      (void)strcpy(name, (*conns)[cdfid]->das().get_table(q)->get_name(p).c_str());

    return NC_NOERR;
}

int
ncattname( int cdfid, int varid, int attnum, char *name)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncattname( (*conns)[cdfid]->ncid(), varid, attnum, name);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattname" ;

    // varid = -1 is used for global atrributes
    if(( varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) {
	    NCadvise(NC_ENOTVAR, "%d is not a valid variable id", varid) ;
	    return(-1);
    }
 
    Pix q = get_das_table(cdfid,varid); // get das table

    // from the attribute table get the requested atrribute name
    Pix p = (*conns)[cdfid]->das().get_table(q)->first_attr();
    for (int i=0; (i < attnum) && p ; i++)
	(*conns)[cdfid]->das().get_table(q)->next_attr(p);

    if(p == (Pix)NULL) // attnum is greater than the attributes in the table
	{
	    NCadvise(NC_ENOTATT, "%d is not a valid attribute id", attnum) ;
	    return(-1) ;
	}
    else
	(void)strcpy(name, (*conns)[cdfid]->das().get_table(q)->get_name(p).c_str());

    return(attnum) ;
}

int 
nc_inq_attid(int cdfid, int varid, const char *name, int *attnump)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_inq_attid((*conns)[cdfid]->ncid(), varid, name, attnump);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if(( varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) 
      return NC_ENOTVAR;

    Pix q = get_das_table(cdfid,varid); // get das table

    // from the attribute table get the requested atrribute name
    Pix p = (*conns)[cdfid]->das().get_table(q)->first_attr();
 
    for (int i = 0; p ; i++){
      if(strcmp((*conns)[cdfid]->das().get_table(q)->get_name(p).c_str(),
		name) == 0){
	if(attnump != 0)
	   *attnump = i;
	return NC_NOERR;
      }
      (*conns)[cdfid]->das().get_table(q)->next_attr(p);
    }
    
    return NC_ENOTATT;
}

int
nc_inq_att(int cdfid, int varid, const char *name, 
	   nc_type *datatypep, size_t *lenp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_inq_att( (*conns)[cdfid]->ncid(), varid, 
			   name, datatypep, lenp);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if(( varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars()))
      return NC_ENOTVAR;

    Pix q = get_das_table(cdfid, varid); // get das table

    // from the attribute table get the requested atrribute by name
    string attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name);

    // from the attribute table get the requested atrribute type by name
    string attrT = (*conns)[cdfid]->das().get_table(q)->get_type(name);

    // from the attribute table get the requested atrribute length by name
    unsigned int attr_siz = (*conns)[cdfid]->das().get_table(q)->get_attr_num(name);

    if(attrV == "") 
	return NC_ENOTATT;
    else { 
	if(datatypep != 0) {
	    if (attrT == "String")
		*datatypep =  NC_CHAR;
	  
	    if (attrT == "Byte")
		*datatypep =  NC_BYTE;
	  
	    if ((attrT == "Int32")||(attrT == "UInt32"))
		*datatypep =  NC_LONG;

	    if ((attrT == "Int16")||(attrT == "UInt16"))
		*datatypep =  NC_SHORT;

	    if (attrT == "Float32")
		*datatypep =  NC_FLOAT;
	  
	    if (attrT == "Float64")
		*datatypep =  NC_DOUBLE;
	}

	if(lenp != 0) {
	    *lenp = 0;
	    if(attrT == "String"){ // string lenght is the total number of
				   // bytes plus terminating character. There
				   // is a '\n' at the end of each string, but
				   // a '\0' at the end of the attibutes.
		for (int i =0; i < attr_siz; i++){
		    attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name,i);
                    *lenp +=  attrV.length() - 1; // EOS included, quotes removed 
		}
		if((*conns)[cdfid]->das().get_table(q)->get_attr_num(name) == 1)
		  *lenp -= 1;   // single string (coming from a netCDF server)
		                  // remove EOS for compatibility
	    }
	    else 
		*lenp =  (size_t) attr_siz;
	}
	
    }

    return NC_NOERR ;
}

int 
nc_inq_attlen(int cdfid, int varid, const char *name, size_t *lenp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_inq_attlen( (*conns)[cdfid]->ncid(), varid, 
			   name, lenp);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if(( varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars()))
      return NC_ENOTVAR;

    Pix q = get_das_table(cdfid, varid); // get das table

    // from the attribute table get the requested atrribute by name
    string attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name);

    // from the attribute table get the requested atrribute type by name
    string attrT = (*conns)[cdfid]->das().get_table(q)->get_type(name);

    // from the attribute table get the requested atrribute length by name
    unsigned int attr_siz = (*conns)[cdfid]->das().get_table(q)->get_attr_num(name);

    if(attrV == "") 
	return NC_ENOTATT;
    else { 
	if(lenp != 0) {
	    *lenp = 0;
	    if(attrT == "String"){ // string lenght is the total number of
				   // bytes plus terminating character. There
				   // is a '\n' at the end of each string, but
				   // a '\0' at the end of the attibutes.
		for (int i =0; i < attr_siz; i++){
		    attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name,i);
                    *lenp +=  attrV.length() - 1; // EOS included, quotes removed 
		}
		if((*conns)[cdfid]->das().get_table(q)->get_attr_num(name) == 1)
		  *lenp -= 1;   // single string (coming from a netCDF server)
		                  // remove EOS for compatibility
	    }
	    else 
		*lenp =  (size_t) attr_siz;
	}
	
    }

    return NC_NOERR ;
}

int 
nc_inq_atttype(int cdfid, int varid, const char *name, nc_type *datatypep)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_inq_atttype( (*conns)[cdfid]->ncid(), varid, 
			   name, datatypep);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if(( varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars()))
      return NC_ENOTVAR;

    Pix q = get_das_table(cdfid, varid); // get das table

    // from the attribute table get the requested atrribute by name
    string attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name);

    // from the attribute table get the requested atrribute type by name
    string attrT = (*conns)[cdfid]->das().get_table(q)->get_type(name);

    if(attrV == "") 
	return NC_ENOTATT;
    else { 
	if(datatypep != 0) {
	    if (attrT == "String")
		*datatypep =  NC_CHAR;
	  
	    if (attrT == "Byte")
		*datatypep =  NC_BYTE;
	  
	    if ((attrT == "Int32")||(attrT == "UInt32"))
		*datatypep =  NC_LONG;

	    if ((attrT == "Int16")||(attrT == "UInt16"))
		*datatypep =  NC_SHORT;

	    if (attrT == "Float32")
		*datatypep =  NC_FLOAT;
	  
	    if (attrT == "Float64")
		*datatypep =  NC_DOUBLE;
	}

    }
	
    return NC_NOERR ;
}

int
ncattinq( int cdfid, int varid, const char *name, 
	  nc_type *datatypep, int *countp)  // Version 2 style returns
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncattinq( (*conns)[cdfid]->ncid(), varid, 
			   name, datatypep, countp);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattinq" ;

    // varid = -1 is used for global atrributes
    if(( varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars()))
	{
	    NCadvise(NC_ENOTVAR, "%d is not a valid variable id", varid) ;
	    return(-1) ;
	}

    Pix q = get_das_table(cdfid, varid); // get das table

    // from the attribute table get the requested atrribute by name
    string attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name);

    // from the attribute table get the requested atrribute type by name
    string attrT = (*conns)[cdfid]->das().get_table(q)->get_type(name);

    // from the attribute table get the requested atrribute length by name
    unsigned int attr_siz = (*conns)[cdfid]->das().get_table(q)->get_attr_num(name);

    if(attrV == "") {
	NCadvise(NC_ENOTATT, "%s attribute does not exist", name) ;
	return(-1) ;
    }
    else { 
	if(datatypep != 0) {
	    if (attrT == "String")
		*datatypep =  NC_CHAR;
	  
	    if (attrT == "Byte")
		*datatypep =  NC_BYTE;

	    if ((attrT == "Int32")||(attrT == "UInt32")) 
		*datatypep =  NC_LONG;

	    if ((attrT == "Int16")||(attrT == "UInt16"))
		*datatypep =  NC_SHORT;
  
	    if (attrT == "Float32")
		*datatypep =  NC_FLOAT;
	  
	    if (attrT == "Float64")
		*datatypep =  NC_DOUBLE;
	}

	if(countp != 0) {
	    *countp = 0;
	    if(attrT == "String"){ // string lenght is the total number of
				   // bytes plus terminating character. There
				   // is a '\n' at the end of each string, but
				   // a '\0' at the end of the attibutes.
		for (int i =0; i < attr_siz; i++){
		    attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name,i);
                    *countp +=  attrV.length() - 1; // EOS included, quotes removed 
		}
		if((*conns)[cdfid]->das().get_table(q)->get_attr_num(name) == 1)
		  *countp -= 1;   // single string (coming from a netCDF server)
		                  // remove EOS for compatibility
	    }
	    else {
		*countp =  attr_siz;
	    }
	}
	
    }

    return(1) ;
}

int
nc_copy_att(int incdf, int invar, const char *name, int outcdf, int outvar)
{
    char *values;         // Attribute vector to be copied,
    nc_type datatype;    // it's type, 
    int count;          // and it's length.

    assert(conns); 

    // does the file id exist?
    if (!conns || incdf < 0 || incdf > MAX_NC_OPEN 
	|| outcdf < 0 || outcdf > MAX_NC_OPEN 
	|| (*conns)[incdf] == NULL || (*conns)[outcdf] == NULL) 
	return NC_EBADID;

    // case 1, both input and output files are local

#ifdef LOCAL
    if((*conns)[incdf]->is_local() && (*conns)[outcdf]->is_local()) {
	rcode = lnc_copy_att((*conns)[incdf]->ncid(), invar, 
			   name, (*conns)[outcdf]->ncid(), outvar);
	return rcode;
    }
#endif


    // case 2, the output file is not local, Error writing over the net.

    if(!(*conns)[outcdf]->is_local()) 
      return NC_EPERM; // DODS network interface is read-only

    // last case, input is from DODS file and output a local netcdf file

#ifdef LOCAL

    // invar = -1 is used for global atrributes
    if(( invar+1 < 0) || (invar+1 > (*conns)[incdf]->nvars())) 
	    return NC_ENOTVAR;

    Pix q = get_das_table(incdf,invar); // get das table

    // from the attribute table get the requested atrribute by name
    string attrV = (*conns)[incdf]->das().get_table(q)->get_attr(name);

    if(attrV == "") 
      return NC_ENOTATT;

    // get the attribute in the right format
    ncattinq(incdf, invar, name, &datatype, &count);

    values = new char [((unsigned)count + 1) * nctypelen(datatype)];
    if (!values) 
      return NC_ENOMEM; // Memory allocation failure 

    correct_att_type(incdf, q, name, (void *)values);

    // now write it to the local file
    rcode = lncattput((*conns)[outcdf]->ncid(), outvar,  
		      name, datatype, count, (void *)values);

    delete [] values;
    return rcode;

#else

    return NC_EPERM;

#endif /* Last case LOCAL */
}

int
ncattcopy(int incdf, int invar, const char *name, int outcdf, int outvar)
{
    char *values;         // Attribute vector to be copied,
    nc_type datatype;    // it's type, 
    int count;          // and it's length.

    assert(conns); 

    // does the file id exist?
    if (!conns || incdf < 0 || incdf > MAX_NC_OPEN 
	|| outcdf < 0 || outcdf > MAX_NC_OPEN 
	|| (*conns)[incdf] == NULL || (*conns)[outcdf] == NULL) 
	return NC_EBADID;

    // case 1, both input and output files are local

#ifdef LOCAL
    if((*conns)[incdf]->is_local() && (*conns)[outcdf]->is_local()) {
	rcode = lncattcopy((*conns)[incdf]->ncid(), invar, 
			   name, (*conns)[outcdf]->ncid(), outvar);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattcopy" ;

    // case 2, the output file is not local, Error writing over the net.

    if(!(*conns)[outcdf]->is_local()) {
	NCadvise(NC_EPERM, "DODS network interface is read-only") ;
	return(-1);
    }

    // last case, input is from DODS file and output a local netcdf file

#ifdef LOCAL

    // invar = -1 is used for global atrributes
    if(( invar+1 < 0) || (invar+1 > (*conns)[incdf]->nvars())) {
	    NCadvise(NC_ENOTVAR, "%d is not a valid variable id", invar) ;
	    return(-1) ;
    }

    Pix q = get_das_table(incdf,invar); // get das table

    // from the attribute table get the requested atrribute by name
    string attrV = (*conns)[incdf]->das().get_table(q)->get_attr(name);

    if(attrV == "") {
	    NCadvise(NC_ENOTATT, "%s attribute does not exist", name) ;
	    return(-1) ;
    }

    // get the attribute in the right format
    ncattinq(incdf, invar, name, &datatype, &count);

    values = new char [((unsigned)count + 1) * nctypelen(datatype)];
    if (!values) 
	return (-1);

    correct_att_type(incdf, q, name, (void *)values);

    // now write it to the local file
    rcode = lncattput((*conns)[outcdf]->ncid(), outvar,  
		      name, datatype, count, (void *)values);

    delete [] values;
    return rcode;

#else

    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);

#endif /* Last case LOCAL */
}

int
ncattget(int cdfid, int varid, const char *name, void *values)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lncattget((*conns)[cdfid]->ncid(), varid, name, values);
	return rcode;
    }
#endif

    cdf_routine_name = "ncattget" ;

    // varid = -1 is used for global atrributes
    if ((varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) {
	NCadvise(NC_ENOTVAR, "%d is not a valid variable id", varid) ;
	return(-1) ;
    }

    Pix q = get_das_table(cdfid,varid); // get das table

    // from the attribute table get the requested atrribute by name
    string attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name);

    if(attrV == "") {
	NCadvise(NC_ENOTATT, "%s attribute does not exist", name) ;
	return(-1) ;
    }
    else
	correct_att_type(cdfid, q, name, values);

    return(1) ;
}

int
nc_get_att_text(int cdfid, int varid, const char *name, char *str)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_att_text((*conns)[cdfid]->ncid(), varid, name, str);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if ((varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) 
      return NC_ENOTVAR;

    char *values;         // Attribute vector to be copied,
    nc_type datatype;    // it's type, 
    size_t count;       // and it's length.

    rcode = nc_inq_att(cdfid, varid, name, &datatype, &count);

    if(rcode != NC_NOERR)
      return rcode;

    if(count == 0)
      return NC_NOERR;

    if(datatype != NC_CHAR)
      return NC_ECHAR;

    values = new char [(count + 1) * nctypelen(datatype)];

    Pix q = get_das_table(cdfid,varid); // get das table
    correct_att_type(cdfid, q, name, (void *)values);

    //    const void *xp = values;   
    rcode = putn_into_text((void **)&values, count, str);

    delete [] values;
    return rcode;
}
 
int
nc_get_att_schar(int cdfid, int varid, const char *name, signed char *tp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_att_schar((*conns)[cdfid]->ncid(), varid, name, tp);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if ((varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) 
      return NC_ENOTVAR;

    char *values;         // Attribute vector to be copied,
    nc_type datatype;    // it's type, 
    size_t count;          // and it's length.

    rcode = nc_inq_att(cdfid, varid, name, &datatype, &count);
  
    if(rcode != NC_NOERR)
      return rcode;

    if(count == 0)
      return NC_NOERR;

    if(datatype == NC_CHAR)
      return NC_ECHAR;

    values = new char [(count + 1) * nctypelen(datatype)];

    Pix q = get_das_table(cdfid,varid); // get das table
    correct_att_type(cdfid, q, name, (void *)values);

    //    const void *xp = values;
    rcode = putn_into_schar((void **)&values, count, tp, datatype);

    delete [] values;
    return rcode;
}

int
nc_get_att_uchar(int cdfid, int varid, const char *name, unsigned char *tp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_att_uchar((*conns)[cdfid]->ncid(), varid, name, tp);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if ((varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) 
      return NC_ENOTVAR;

    char *values;         // Attribute vector to be copied,
    nc_type datatype;    // it's type, 
    size_t count;          // and it's length.

    rcode = nc_inq_att(cdfid, varid, name, &datatype, &count);

   if(rcode != NC_NOERR)
      return rcode;

    if(count == 0)
      return NC_NOERR;

    if(datatype == NC_CHAR)
      return NC_ECHAR;

    values = new char [(count + 1) * nctypelen(datatype)];

    Pix q = get_das_table(cdfid,varid); // get das table
    correct_att_type(cdfid, q, name, (void *)values);

    //    const void *xp = values;
    rcode = putn_into_uchar((void **)&values, count, tp, datatype);

    delete [] values;
    return rcode;
}

int
nc_get_att_short(int cdfid, int varid, const char *name, short *tp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_att_short((*conns)[cdfid]->ncid(), varid, name, tp);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if ((varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) 
      return NC_ENOTVAR;

    char *values;         // Attribute vector to be copied,
    nc_type datatype;    // it's type, 
    size_t count;          // and it's length.

    rcode = nc_inq_att(cdfid, varid, name, &datatype, &count);

    if(rcode != NC_NOERR)
      return rcode;

    if(count == 0)
      return NC_NOERR;

    if(datatype == NC_CHAR)
      return NC_ECHAR;

    values = new char [(count + 1) * nctypelen(datatype)];

    Pix q = get_das_table(cdfid,varid); // get das table
    correct_att_type(cdfid, q, name, (void *)values);

    //    const void *xp = values;
    rcode = putn_into_short((void **)&values, count, tp, datatype);

    delete [] values;
    return rcode;
}

int
nc_get_att_int(int cdfid, int varid, const char *name, int *tp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_att_int((*conns)[cdfid]->ncid(), varid, name, tp);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if ((varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) 
      return NC_ENOTVAR;

    char *values;         // Attribute vector to be copied,
    nc_type datatype;    // it's type, 
    size_t count;          // and it's length.

    rcode = nc_inq_att(cdfid, varid, name, &datatype, &count);

    if(rcode != NC_NOERR)
      return rcode;

    if(count == 0)
      return NC_NOERR;

    if(datatype == NC_CHAR)
      return NC_ECHAR;

    values = new char [(count + 1) * nctypelen(datatype)];

    Pix q = get_das_table(cdfid,varid); // get das table
    correct_att_type(cdfid, q, name, (void *)values);

    //    const void *xp = values;
    rcode = putn_into_int((void **)&values, count, tp, datatype);

    delete [] values;
    return rcode;
}

int
nc_get_att_long(int cdfid, int varid, const char *name, long *tp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_att_long((*conns)[cdfid]->ncid(), varid, name, tp);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if ((varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) 
      return NC_ENOTVAR;

    char *values;         // Attribute vector to be copied,
    nc_type datatype;    // it's type, 
    size_t count;          // and it's length.

    rcode = nc_inq_att(cdfid, varid, name, &datatype, &count);
    
    if(rcode != NC_NOERR)
      return rcode;

    if(count == 0)
      return NC_NOERR;

    if(datatype == NC_CHAR)
      return NC_ECHAR;

    values = new char [(count + 1) * nctypelen(datatype)];

    Pix q = get_das_table(cdfid,varid); // get das table
    correct_att_type(cdfid, q, name, (void *)values);

    //    const void *xp = values;
    rcode = putn_into_long((void **)&values, count, tp, datatype);

    delete [] values;
    return rcode;
}

int
nc_get_att_float(int cdfid, int varid, const char *name, float *tp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_att_float((*conns)[cdfid]->ncid(), varid, name, tp);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if ((varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) 
      return NC_ENOTVAR;

    char *values;         // Attribute vector to be copied,
    nc_type datatype;    // it's type, 
    size_t count;          // and it's length.

    rcode = nc_inq_att(cdfid, varid, name, &datatype, &count);

    if(rcode != NC_NOERR)
      return rcode;

    if(count == 0)
      return NC_NOERR;

    if(datatype == NC_CHAR)
      return NC_ECHAR;

    values = new char [(count + 1) * nctypelen(datatype)];

    Pix q = get_das_table(cdfid,varid); // get das table
    correct_att_type(cdfid, q, name, (void *)values);

    //    const void *xp = values;
    rcode = putn_into_float((void **)&values, count, tp, datatype);

    delete [] values;
    return rcode;
}

int
nc_get_att_double(int cdfid, int varid, const char *name, double *tp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_att_double((*conns)[cdfid]->ncid(), varid, name, tp);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if ((varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) 
      return NC_ENOTVAR;

    char *values;         // Attribute vector to be copied,
    nc_type datatype;    // it's type, 
    size_t count;          // and it's length.

    rcode = nc_inq_att(cdfid, varid, name, &datatype, &count);
    
    if(rcode != NC_NOERR)
      return rcode;

    if(count == 0)
      return NC_NOERR;

    if(datatype == NC_CHAR)
      return NC_ECHAR;

    values = new char [(count + 1) * nctypelen(datatype)];

    Pix q = get_das_table(cdfid,varid); // get das table
    correct_att_type(cdfid, q, name, (void *)values);

    //    const void *xp = values;
    rcode = putn_into_double((void **)&values, count, tp, datatype);

    delete [] values;
    return rcode;
}

int
nc_get_att(int cdfid, int varid, const char *name, void *values)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_att((*conns)[cdfid]->ncid(), varid, name, values);
	return rcode;
    }
#endif

    // varid = -1 is used for global atrributes
    if ((varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars())) 
	return NC_ENOTVAR;

    Pix q = get_das_table(cdfid,varid); // get das table

    // from the attribute table get the requested atrribute by name
    string attrV = (*conns)[cdfid]->das().get_table(q)->get_attr(name);

    if(attrV == "") 
	return NC_ENOTATT;
    else
	correct_att_type(cdfid, q, name, values);

    return NC_NOERR;
}

// $Log: Dattr.cc,v $
// Revision 1.5.4.1  2004/06/15 18:43:39  jimg
// Repaired code in void correct_att_type(). The code did not test for the
// attribute types unsigned int16 or uint32. This may not actually be a problem
// in practice with this code, but in version on the trunk the function does
// need to be fixed... See bug 714.
//
// Revision 1.5  2003/01/28 07:08:24  jimg
// Merged with release-3-2-8.
//
// Revision 1.4.4.3  2002/12/18 23:40:33  pwest
// gcc3.2 compile corrections, mainly regarding using statements. Also,
// problems with multi line string literatls.
//
// Revision 1.4.4.2  2002/12/05 20:32:39  pwest
// Corrected problems with IteratorAdapter code and cleaned up file descriptors
// and memory management problems.
//
// Revision 1.4.4.1  2002/11/18 01:11:37  rmorris
// VC++ required an explicit cast to Pix in certain cases.
//
// Revision 1.4  2000/10/06 01:22:02  jimg
// Moved the CVS Log entries to the ends of files.
// Modified the read() methods to match the new definition in the dap library.
// Added exception handlers in various places to catch exceptions thrown
// by the dap library.
//
// Revision 1.3  1999/11/05 05:15:03  jimg
// Result of merge woth 3-1-0
//
// Revision 1.1.2.2  1999/10/29 05:05:19  jimg
// Reza's fixes plus the configure & Makefile update
//
// Revision 1.2  1999/10/21 13:19:05  reza
// IMAP and other bug fixed for version3.
//
// Revision 1.1.2.1  1999/10/15 19:50:55  jimg
// Changed return values and conditions for NC API entry points
//
// Revision 1.1  1999/07/28 00:22:40  jimg
// Added
//
// Revision 1.15  1999/05/07 23:45:30  jimg
// String --> string fixes
//
// Revision 1.14  1999/03/30 05:20:55  reza
// Added support for the new data types (Int16, UInt16, and Float32).
//
// Revision 1.13  1997/03/26 05:44:20  reza
// GLOBAL attribute can now be named anything with "_GLOBAL" in it (i.e.
// HDF_GLOBAL, MAT_GLOBAL, DSP_GLOBAL, etc.). This should improve interoperability
// with non-netCDF servers. Also, the DAS no longer requires attributes for all
// the variables. An internal empty attribute table (NC_BLANK) manages missing ones.
//
// Revision 1.12  1997/03/25 16:05:56  reza
// Fixed CVS duplicate log reports.
//
// Revision 1.11  1996/09/17 00:26:04  jimg
// Merged changes from a side branch which contained various changes from
// Reza and Charles.
// Removed ncdump and netexec since ncdump is now in its own directory and
// netexec is no longer used.
//
// Revision 1.10.2.1  1996/06/25 22:04:04  jimg
// Version 2.0 from Reza.
//
// Revision 1.9.2.2  1996/06/10 19:36:06  jimg
// Replaced strcpy in correct_att_type() with memcpy.
//
// Revision 1.9.2.1  1996/04/02 19:29:53  jimg
// Changed the `conns' variable so that now it is a global *pointer*, not an
// object. Each call that might add to the conns object must first check to see
// that it has been created. This was done because of link-time problems with
// static global objects.
// Fixed up some comment stuff.
// Added assert() calls for conns.
//
// Revision 1.9  1995/07/09  21:33:29  jimg
// Added copyright notice.
//
// Revision 1.8  1995/06/29  15:42:02  jimg
// Fixed instances of delete which should have included brackets
//
// Revision 1.7  1995/06/28  20:22:33  jimg
// Replaced malloc calls with calls to new (and calls to free with calls to
// delete).
//
// Revision 1.6  1995/06/23  13:59:32  reza
// Minor changes to make Efence work.
//
// Revision 1.5  1995/03/16  16:38:36  reza
// Updated byte transfer. Bytes are no longer transmitted in binary but in
// an ASCII string (0-255).
//
// Revision 1.4  1994/12/22  04:46:18  reza
// Updated to use DODS new attribute array capability.
//
// Revision 1.3  1994/12/08  19:01:27  reza
// Added support for local netCDF file access through the original netCDF code.
//
// Revision 1.2  1994/11/23  21:05:50  reza
// First working version.
//
// Revision 1.1  1994/11/03  04:35:44  reza
// Preliminary version of netCDF -> DODS library.
