// (c) COPYRIGHT URI/MIT 1994-1996
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      reza            Reza Nekovei (reza@ieee.org)

// DODS-netCDF surrogate library functions for file handling. Note that with
// the preprocessor symbol LOCAL defined this library will access local files
// without using the DODS network services. Defining LOCAL will add support
// for local files through a slightly modifed version of the original netCDF
// library. This library has the same functionality as the original (it can
// access only files on a local hard disk) but has renamed entry points to
// avoid linker conflicts.
// 
// ReZa 10/13/94
//
// Rewrite to support netCDF version 3 interface; 5/12/99, Reza
//
#include "config_dap.h"

static char rcsid[] not_used ={"$Id: Dnc.cc,v 1.8.2.4 2003/10/09 05:59:03 reza Exp $"};

#ifdef __GNUG__
#pragma implementation
#endif

#include <assert.h>
#include <string>

#include "NCStructure.h"
#include "Dnetcdf.h"
#include "Connect.h"
#include "NCConnect.h"

// Make these symbols external so as to avoid link-time conflicts with their
// definitions in the localized version of the original
// implementation of netcdf.

#ifdef LOCAL
extern char *cdf_routine_name;
extern int ncopts;
extern int ncerr;
#else
char *cdf_routine_name;
int ncopts = NC_VERBOSE;
int ncerr;
#endif

int rcode;

Connections<NCConnect *> *conns = 0;


int
nc__create_mp(const char * path, int ioflags, size_t initialsz, int basepe,
	size_t *chunksizehintp, int *ncid_ptr)
{

    NCConnect *c = new NCConnect(path);

#ifdef LOCAL
    if(c->is_local()) {  
      rcode = lnc__create_mp(path, ioflags, initialsz, basepe,
			    chunksizehintp, ncid_ptr);
	if (rcode == NC_NOERR) {
	    c->set_ncid(*ncid_ptr);
	    if (!conns)
		conns = new Connections<NCConnect *>(MAX_NC_OPEN);
	    *ncid_ptr = conns->add_connect(c);
	    return NC_NOERR;
	}
	else {
	  //tmp_out	    delete c;
	    return rcode;
	}
    }
#endif

    //tmp_out    delete c;
    return NC_EPERM;
}

int
nc__create(const char * path, int ioflags, size_t initialsz,
	size_t *chunksizehintp, int *ncid_ptr)
{
	return nc__create_mp(path, ioflags, initialsz, 0,
		chunksizehintp, ncid_ptr);
}


int
nc_create(const char * path, int ioflags, int *ncid_ptr)
{
        return nc__create(path, ioflags, 0, NULL, ncid_ptr);
}


int
nccreate(const char *path, int cmode)
{
    int id;
    NCConnect *c = new NCConnect(path);

#ifdef LOCAL
    if(c->is_local()) {  
	rcode = lnccreate(path, cmode);
	if (rcode != -1) {
	    c->set_ncid(rcode);
	    if (!conns)
		conns = new Connections<NCConnect *>(MAX_NC_OPEN);
	    id = conns->add_connect(c);
	    return id;
	}
	else {
	  //tmp_out	    delete c;
	    return rcode;
	}
    }
#endif

    cdf_routine_name = "nccreate" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;

    //tmp_out    delete c;
    return(-1);
}

int
nc__open(const char * path, int ioflags,
	size_t *chunksizehintp, int *ncid_ptr)
{
	return nc__open_mp(path, ioflags, 0,
		chunksizehintp, ncid_ptr);
}

int
nc__open_mp(const char * urlpath, int ioflags, int basepe,
	size_t *chunksizehintp, int *ncid_ptr)
{
    string path = urlpath;
    string var_name;

    size_t q_mark = path.find("?");
    string ce = "";
    if (q_mark != string::npos) {
	ce = path.substr(q_mark + 1);
	path = path.substr(0, q_mark);
    }

    NCConnect *c = new NCConnect(path);

#ifdef LOCAL
    if (c->is_local()) {  
	try {
	    rcode = lnc__open_mp(path.c_str(), ioflags, basepe, 
				 chunksizehintp, ncid_ptr);
	}
	catch (Error &e) {
	    delete c ;
	    return rcode;
	}

	if (rcode == NC_NOERR) {
	    c->set_ncid(*ncid_ptr);
	    if (!conns) {
		conns = new Connections<NCConnect *>(MAX_NC_OPEN);
	    }
	    *ncid_ptr = conns->add_connect(c);
	    return NC_NOERR;
	} else {
	    delete c ;
	    return rcode;
	}
    }
#endif

    // !_CRAYMPP, only pe 0 is valid
    if(basepe != 0)
      return NC_EINVAL;

    // Read only network access
    if (ioflags != NC_NOWRITE)
      return NC_EPERM;

    try {
	c->request_das( ) ;
	c->request_dds( ) ;
	c->dds().parse_constraint(ce);
	c->request_Cdds(ce);
    }
    catch (Error &e) {
      if(e.get_error_code() != NC_NOERR)
	return e.get_error_code(); 
      else
	return NC_EINVAL;
    }

    // *** I think this code invalidates the 'Pix' iterator 12/23/02 jhrg
    // Remove all the non-supported types for now
    Pix Cp=c->dds().first_var();
    //    if((c->dds().var(Cp)->type() != dods_grid_c )&& 
    //   (c->dds().var(Cp)->type() != dods_array_c )){ 
    if( ( c->dds().var(Cp)->type() == dods_structure_c ) ||
        ( c->dds().var(Cp)->type() == dods_sequence_c ) || 
        ( c->dds().var(Cp)->type() == dods_list_c ))
    {
	var_name = c->dds().var(Cp)->name();
	/* vector erase, iterator points to next element, dont call next_var */
	//c->dds().next_var(Cp);
	c->dds().del_var(var_name);
	c->Cdds().del_var(var_name);
	//c->das().del_attr(var_name);
    } else {
	c->dds().next_var(Cp);
    }

    // *** I think this code invalidates the 'Pix' iterator 12/23/02 jhrg
    // *** Can the if above be combined with this while? 12/23/02 jhrg
    while (Cp) {
	if ( ( c->dds().var(Cp)->type() == dods_structure_c ) ||
	     ( c->dds().var(Cp)->type() == dods_sequence_c ) || 
	     ( c->dds().var(Cp)->type() == dods_list_c ))
	{
	    var_name = c->dds().var(Cp)->name();
	    /* vector erase, iter points to next element, dont call next_var */
	    //c->dds().next_var(Cp);
	    c->dds().del_var(var_name);
	    c->Cdds().del_var(var_name);
	} else {
	    c->dds().next_var(Cp);	
	}
    }

    // well now we can extract dim. and attr. info. into tables
    c->parse_das_loc();

    c->parse_dims();

    if (!conns) {
	conns = new Connections<NCConnect *>(MAX_NC_OPEN);
    }
    *ncid_ptr = conns->add_connect(c);

    return NC_NOERR;
}


int
nc_open(const char * path, int ioflags, int *ncid_ptr)
{
        return nc__open(path, ioflags, NULL, ncid_ptr);
}


int
ncopen(const char *urlpath, int cmode)
{
    int id;
    string path = urlpath;
    bool status = true;
    string var_name;

    size_t q_mark = path.find("?");
    string ce = "";
    if (q_mark != string::npos) {
	ce = path.substr(q_mark + 1);
	path = path.substr(0, q_mark);
    }

    NCConnect *c = new NCConnect(path);

#ifdef LOCAL
    if (c->is_local()) {  
	try {
	    rcode = lncopen(path.c_str(), cmode);
	}
	catch (Error &e) {
	    return rcode;
	}

	if (rcode != -1) {
	    c->set_ncid(rcode);
	    if (!conns)
		conns = new Connections<NCConnect *>(MAX_NC_OPEN);
	    id = conns->add_connect(c);
	    return id;
	} else {
	    return rcode;
	}
    }
#endif

    cdf_routine_name = "ncopen";

    if (cmode != NC_NOWRITE) {    
	NCadvise(NC_EPERM, 
		 "Bad file mode, Only read access allowed over the network");
	return -1;
    }

    try { c->request_das(); }
    catch (Error &e) {

      if(e.get_error_code() != NC_NOERR) 
	if(e.get_error_code() < 1000) //DODS internal error values are above 1000
	  NCadvise(e.get_error_code(), "filename `%s'", path.c_str()); //netCDF/System errors
	else 
	  NCadvise(NC_EINVAL,"Error in getting DAS from `%s': `%s'", path.c_str(), e.get_error_message().c_str()); //DODS internal errors

      return -1;
    }

    try { c->request_dds(); }
    catch (Error &e) {

      if(e.get_error_code() != NC_NOERR) 
	if(e.get_error_code() < 1000) //DODS internal error values are above 1000
	  NCadvise(e.get_error_code(), "filename `%s'", path.c_str());
	else 
	  NCadvise(NC_EINVAL,"Error in getting DDS from `%s': `%s'", path.c_str(), e.get_error_message().c_str());
      
      return -1;
    }

    try { c->dds().parse_constraint(ce);}
    catch (Error &e) {

      if(e.get_error_code() != NC_NOERR) 
	if(e.get_error_code() < 1000) //DODS internal error values 
	  NCadvise(e.get_error_code(), "filename `%s'", path.c_str());
	else 
	  NCadvise(NC_EINVAL,"Error in  parsing constarint from `%s': `%s'", path.c_str(), e.get_error_message().c_str());
      
      return -1;
    }

    try { c->request_Cdds(ce); }
    catch (Error &e) {
 
     if(e.get_error_code() != NC_NOERR) 
       if(e.get_error_code() < 1000) //DODS internal error values are above 1000
	 NCadvise(e.get_error_code(), "filename `%s'", path.c_str());
       else 
	 NCadvise(NC_EINVAL,"Error in getting CDDS from `%s': `%s'", path.c_str(), e.get_error_message().c_str());

     return -1;
    }

#ifdef TEST
    // Use variable rename to resolve structures
    for (Pix Sp=c->dds().first_var(); Sp; c->dds().next_var(Sp))
    {
	if (c->dds().var(Sp)->type() == dods_structure_c) {
	    NCStructure *stq = (NCStructure *) c->dds().var(Sp);
	    for (Pix p=stq->first_var(); p!=0; stq->next_var(p))
	    if( ( stq->var(p)->type() == dods_grid_c ) ||
	        ( stq->var(p)->type() == dods_array_c ) )
	    {
		//
		stq->var(p)->set_name(stq->name() + "." + stq->var(p)->name()); 

		//
		c->dds().add_var(stq->var(p));
		c->Cdds().add_var(stq->var(p));
	    }
	}
    }

    c->dds().print();
#endif

    // *** I think this code invalidates the 'Pix' iterator 12/23/02 jhrg
    Pix Cp=c->dds().first_var();
    if((c->dds().var(Cp)->type() == dods_structure_c ) ||
       (c->dds().var(Cp)->type() == dods_sequence_c ) || 
       (c->dds().var(Cp)->type() == dods_list_c )){ 

      var_name = c->dds().var(Cp)->name();
      /* vector erase, iterator points to next element, dont call next_var */
      //c->dds().next_var(Cp);
      c->dds().del_var(var_name);
      c->Cdds().del_var(var_name);
    }
    else
      c->dds().next_var(Cp);

    while(Cp) {

    if((c->dds().var(Cp)->type() == dods_structure_c ) ||
       (c->dds().var(Cp)->type() == dods_sequence_c ) || 
       (c->dds().var(Cp)->type() == dods_list_c )){ 

	var_name = c->dds().var(Cp)->name();
        /* vector erase, iterator points to next element, dont call next_var */
	//c->dds().next_var(Cp);

	c->dds().del_var(var_name);
	c->Cdds().del_var(var_name);
      }
      else
	c->dds().next_var(Cp);
	
    }

#ifdef TEST
    //    c->das().print();
    c->dds().print();
    c->Cdds().print();
#endif

    // well now we can extract dim. and attr. info. into tables
    c->parse_das_loc();
    c->parse_dims();

#ifdef TEST
    for (int i=0 ; i<7 ; i++)
	cerr << i << "-" << c->das_loc(i) << endl;
    c->das().print();    c->dds().next_var(Cp);
    c->dds().print();
#endif

    if (status == true) {
	if (!conns)
	    conns = new Connections<NCConnect *>(MAX_NC_OPEN);

	return(conns->add_connect(c)) ;
    }
    else {
	return -1;
    }
}

int
nc_close(int cdfid)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_close((*conns)[cdfid]->ncid());

	delete (*conns)[cdfid];
	conns->del_connect(cdfid);

	return rcode;
    }
#endif

#ifdef TEST
    (*conns)[cdfid]->das().print();
    (*conns)[cdfid]->dds().print();
#endif

    delete (*conns)[cdfid];

    conns->del_connect(cdfid);
    
    return NC_NOERR;
}


int
ncclose(int cdfid)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncclose((*conns)[cdfid]->ncid());

	delete (*conns)[cdfid];
	conns->del_connect(cdfid);

	return rcode;
    }
#endif

    cdf_routine_name = "ncclose" ;

    delete (*conns)[cdfid];

    conns->del_connect(cdfid);
    
    return(0) ;
}

int
nc_delete(const char * path)
{
	return nc_delete_mp(path, 0);
}

int
nc_delete_mp(const char * path, int basepe)
{
    NCConnect *c = new NCConnect(path);

#ifdef LOCAL
    if(c->is_local()) {  
      rcode = lnc_delete_mp(path, basepe);
      return rcode;
    }
#endif

    return NC_EPERM;
}

int
nc_inq_base_pe(int cdfid, int *pe)
{
  if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
      || (*conns)[cdfid] == NULL) // does the file id exist?
    return NC_EBADID;
 
#ifdef LOCAL
  if((*conns)[cdfid]->is_local()) {
    rcode = lnc_inq_base_pe((*conns)[cdfid]->ncid(), pe);
    return rcode;
  }
#endif
  *pe = 0; /* !_CRAYMPP, only pe 0 is valid */
  return NC_NOERR;
}

int
nc_set_base_pe(int cdfid, int pe)
{
  if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
      || (*conns)[cdfid] == NULL) // does the file id exist?
    return NC_EBADID;
 
#ifdef LOCAL
  if((*conns)[cdfid]->is_local()) {
    rcode = lnc_set_base_pe((*conns)[cdfid]->ncid(), pe);
    return rcode;
  }
#endif
  return NC_EPERM;
}

extern int num_attr(int cdfid, int varid);

int
nc_inq(int cdfid, int *ndimsp, int *nvarsp, int *nattsp, int *xtendimp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_inq((*conns)[cdfid]->ncid(), ndimsp, nvarsp, nattsp, xtendimp);
	return rcode;
    }
#endif

  if(nvarsp != NULL)
    *nvarsp = (*conns)[cdfid]->nvars();

  if(nattsp != NULL)
    *nattsp = num_attr(cdfid,NC_GLOBAL);

  if(ndimsp != NULL)
    *ndimsp = (*conns)[cdfid]->ndims();
		
  if(xtendimp != NULL)
    *xtendimp = (*conns)[cdfid]->recdim(); 
	
  return NC_NOERR;
}


int 
nc_inq_ndims(int cdfid, int *ndimsp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_inq_ndims((*conns)[cdfid]->ncid(), ndimsp);
	return rcode;
    }
#endif

  if(ndimsp != NULL)
    *ndimsp = (*conns)[cdfid]->ndims();
		
  return NC_NOERR;
}


int 
nc_inq_nvars(int cdfid, int *nvarsp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_inq_nvars((*conns)[cdfid]->ncid(), nvarsp);
	return rcode;
    }
#endif

  if(nvarsp != NULL)
    *nvarsp = (*conns)[cdfid]->nvars();

  return NC_NOERR;
}


int 
nc_inq_natts(int cdfid, int *nattsp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_inq_natts((*conns)[cdfid]->ncid(), nattsp);
	return rcode;
    }
#endif

  if(nattsp != NULL)
    *nattsp = num_attr(cdfid,NC_GLOBAL);

  return NC_NOERR;
}


int 
nc_inq_unlimdim(int cdfid, int *xtendimp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_inq_unlimdim((*conns)[cdfid]->ncid(), xtendimp);
	return rcode;
    }
#endif

  if(xtendimp != NULL)
    *xtendimp = (*conns)[cdfid]->recdim(); 
	
  return NC_NOERR;
}


int
ncinquire(int cdfid,int *ndimsp,int *nvarsp, int *nattrsp, int *xtendimp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
  if((*conns)[cdfid]->is_local()) {
    rcode = lncinquire((*conns)[cdfid]->ncid(), ndimsp, nvarsp,
			      nattrsp,xtendimp);
    return rcode;
  }
#endif
  
  cdf_routine_name = "ncinquire" ;

  if(nvarsp != NULL)
    *nvarsp = (*conns)[cdfid]->nvars();

  if(nattrsp != NULL)
    *nattrsp = num_attr(cdfid,NC_GLOBAL);

  if(ndimsp != NULL)
    *ndimsp = (*conns)[cdfid]->ndims();
		
  if(xtendimp != NULL)
    *xtendimp = (*conns)[cdfid]->recdim(); 
	
  return(cdfid) ;
}


int
nc_sync(int cdfid)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode = lnc_sync((*conns)[cdfid]->ncid());
      return rcode;
    }
#endif

    return NC_EPERM;
}


int
ncsync(int cdfid)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncsync((*conns)[cdfid]->ncid());
	return rcode;
    }
#endif

    cdf_routine_name = "ncsync" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;

    return(-1);
}


int
nc_abort(int cdfid)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode = lnc_abort((*conns)[cdfid]->ncid());
      return rcode;
    }
#endif
   rcode = nc_close(cdfid);
   return rcode;
}


int
ncabort(int cdfid)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncabort((*conns)[cdfid]->ncid());
	return rcode;
    }
#endif
    
    cdf_routine_name = "ncabort" ;

    rcode = ncclose(cdfid);

    return rcode;
}


/* 
 * Deprecated function ;
 */
int
ncnobuf(int cdfid)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

    cdf_routine_name = "ncnobuf" ;

    /* NOOP */

    return(0);
}


int
nc_redef(int cdfid)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode = lnc_redef((*conns)[cdfid]->ncid());
      return rcode;
    }
#endif

    return NC_EPERM;
}

  
int
ncredef(int cdfid)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {  
	rcode = lncredef((*conns)[cdfid]->ncid());
	return rcode;
    }
#endif

    cdf_routine_name = "ncredef" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}


int
nc__enddef(int cdfid, size_t h_minfree, size_t v_align,
        size_t v_minfree, size_t r_align)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode = lnc__enddef((*conns)[cdfid]->ncid(), h_minfree,
			  v_align, v_minfree, r_align);
      return rcode;
    }
#endif

    return NC_NOERR; // no effect on a remote file

}


int
nc_enddef(int cdfid)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode = lnc_enddef((*conns)[cdfid]->ncid());
      return rcode;
    }
#endif

    return NC_NOERR; // no effect on a remote file
}


int
ncendef(int cdfid )
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) { 
	rcode = lncendef((*conns)[cdfid]->ncid());
	return rcode;
    }
#endif

    cdf_routine_name = "ncendef" ;

    return(0);
}


int
nc_set_fill(int cdfid, int fillmode, int *old_mode_ptr)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

    if((*conns)[cdfid] == NULL) // does the file id exist?
      return NC_EBADID ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode = lnc_set_fill((*conns)[cdfid]->ncid(), fillmode, old_mode_ptr);
      return rcode;
    }
#endif

    return NC_EPERM;
}


int
ncsetfill(int cdfid, int fillmode)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {  
	rcode = lncsetfill((*conns)[cdfid]->ncid(), fillmode);
	return rcode;
    }
#endif

    cdf_routine_name = "ncsetfill" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;

    return(-1);
}


const char *
nc_inq_libvers(void)
{
#ifdef LOCAL
  return lnc_inq_libvers(); // local netCDF library version (not DODS)
#else
  return ((char *)0);
#endif
}


const char *
nc_strerror(int err)
{
#ifdef LOCAL
  return lnc_strerror(err); 
#endif
  return((char *)0);
}

// $Log: Dnc.cc,v $
// Revision 1.8.2.4  2003/10/09 05:59:03  reza
// Fixed for Fortran compiler.
//
// Revision 1.8.2.3  2003/07/21 07:05:48  reza
// Added support for Cray multi-processor (CRAYMPP) interfaces, they use processor element 0 for all non-CrayMPP platforms. The Fortran interface needs them for a clean build.
//
// Revision 1.8.2.2  2003/06/06 08:23:41  reza
// Updated the servers to netCDF-3 and fixed error handling between client and server.
//
// Revision 1.8.2.1  2003/05/06 13:23:54  reza
// Fixed the netCDF V3.5 upgrade.
//
// Revision 1.8  2003/05/02 16:30:29  jimg
// Fixes for the builds
//
// Revision 1.7  2003/01/28 07:08:24  jimg
// Merged with release-3-2-8.
//
// Revision 1.4.4.9  2002/12/27 00:37:36  jimg
// Removed some unused variables.
// Switched to set_ncid() where appropriate.
//
// Revision 1.4.4.8  2002/12/23 23:06:40  jimg
// Build Fixes.
//
// Revision 1.4.4.7  2002/11/21 23:02:19  pwest
// fixed some memory leaks and memory read errors
//
// Revision 1.4.4.6  2002/11/06 22:53:23  pwest
// Cleaned up some uninitialized memory read warnings from purify
//
// Revision 1.6  2002/05/03 00:01:52  jimg
// Merged with release-3-2-7.
//
// Revision 1.4.4.5  2002/04/17 20:11:32  jimg
// Fixed the return value of ncopen() so that any failure results in a return
// value of -1. This fixes bug 393.
//
// Revision 1.4.4.4  2001/12/26 05:53:53  rmorris
// Force a return value in nc_strerror().  VC++ always requires a return;
//
// Revision 1.4.4.3  2001/12/26 03:39:05  rmorris
// nc_inq_libvers() must return _something_ under win32.  No return
// never allowed.
//
// Revision 1.5  2001/08/30 23:08:23  jimg
// Merged with 3.2.4
//
// Revision 1.4.4.2  2001/08/30 06:23:03  reza
// Fixed bugs reported by bugzilla.
//
// Revision 1.4.4.1  2001/05/17 16:33:57  reza
// Fixed exception handlers to return correct error codes.
//
// Revision 1.4  2000/10/06 01:22:02  jimg
// Moved the CVS Log entries to the ends of files.
// Modified the read() methods to match the new definition in the dap library.
// Added exception handlers in various places to catch exceptions thrown
// by the dap library.
//
// Revision 1.3  1999/11/05 05:15:04  jimg
// Result of merge woth 3-1-0
//
// Revision 1.1.2.2  1999/10/29 05:05:19  jimg
// Reza's fixes plus the configure & Makefile update
//
// Revision 1.2  1999/10/21 13:19:06  reza
// IMAP and other bug fixed for version3.
//
// Revision 1.1.2.1  1999/10/15 19:50:55  jimg
// Changed return values and conditions for NC API entry points
//
// Revision 1.1  1999/07/28 00:22:41  jimg
// Added
