//
// A data exchange interface for DODS-netCDF V3.x API library.
//
//  Reza 5/18/99
// 

// $Id: Dncx.cc,v 1.4 2000/10/06 01:22:02 jimg Exp $ 

#include "Dncx.h"
#include <string.h>
#include <limits.h>
/* alias poorly named limits.h macros */
#define  SHORT_MAX  SHRT_MAX
#define  SHORT_MIN  SHRT_MIN
#define USHORT_MAX USHRT_MAX
#include <float.h>
#ifndef FLT_MAX /* This POSIX macro missing on some systems */
# ifndef NO_IEEE_FLOAT
# define FLT_MAX 3.40282347e+38f
# else
# error "You will need to define FLT_MAX"
# endif
#endif

int
putn_into_uchar(void **xpp, size_t nelems, unsigned char *tp, nc_type type)
{
  //  unsigned char *bp;
  short *sp;
  int *ip; 
  float *fp;
  double *dp;
  int stat = NC_NOERR;

  switch (type) {
    case NC_CHAR:
      return NC_ECHAR;
    case NC_BYTE:
      (void) memcpy(tp, *xpp, nelems);
#ifdef Never
      bp = (unsigned char *) *xpp; 
      for( ; nelems != 0; nelems--, bp++, tp++)
	*tp = (unsigned char) *bp;
#endif
      break;
    case NC_SHORT:
      sp = (short *) *xpp; 
      for( ; nelems != 0; nelems--, sp++, tp++){
        if(*sp > UCHAR_MAX || *sp < 0)
	  stat = NC_ERANGE;
	*tp = (unsigned char) *sp;
      }     
      break;
    case NC_INT:
      ip = (int *) *xpp; 
      for( ; nelems != 0; nelems--, ip++, tp++){
       if(*ip > UCHAR_MAX || *ip < 0)
	  stat = NC_ERANGE;
	*tp = (unsigned char) *ip;
      }  
      break;
    case NC_FLOAT: 
      fp = (float *) *xpp; 
      for( ; nelems != 0; nelems--, fp++, tp++){
	if(*fp > UCHAR_MAX || *fp < 0)
	  stat = NC_ERANGE;      
	*tp = (unsigned char) *fp; 
      }  
      break;
    case NC_DOUBLE:
      dp = (double *) *xpp; 
      for( ; nelems != 0; nelems--, dp++, tp++){
	if(*dp > UCHAR_MAX || *dp < 0)
	  stat = NC_ERANGE;      
	*tp = (unsigned char) *dp;
      }  
      break;
  }

  return stat;
}

int
putn_into_schar(void **xpp, size_t nelems, signed char *tp, nc_type type)
{
 //  signed char *bp;
  short *sp;
  int *ip; 
  float *fp;
  double *dp;
  int stat = NC_NOERR;

  switch (type) {
    case NC_CHAR:
      return NC_ECHAR;
    case NC_BYTE:
      (void) memcpy(tp, *xpp, nelems);
#ifdef Never
      bp = (signed char *) *xpp; 
      for( ; nelems != 0; nelems--, bp++, tp++)
	*tp = (signed char) *bp;
#endif
      break;
    case NC_SHORT:
      sp = (short *) *xpp; 
      for( ; nelems != 0; nelems--, sp++, tp++){
        if(*sp > SCHAR_MAX || *sp < SCHAR_MIN)
	  stat = NC_ERANGE;
	*tp = (signed char) *sp;
      }     
      break;
    case NC_INT:
      ip = (int *) *xpp; 
      for( ; nelems != 0; nelems--, ip++, tp++){
       if(*ip > SCHAR_MAX || *ip < SCHAR_MIN)
	  stat = NC_ERANGE;
	*tp = (signed char) *ip;
      }  
      break;
    case NC_FLOAT: 
      fp = (float *) *xpp; 
      for( ; nelems != 0; nelems--, fp++, tp++){
	if(*fp > SCHAR_MAX || *fp < SCHAR_MIN)
	  stat = NC_ERANGE;      
	*tp = (signed char) *fp; 
      }  
      break;
    case NC_DOUBLE:
      dp = (double *) *xpp; 
      for( ; nelems != 0; nelems--, dp++, tp++){
	if(*dp > SCHAR_MAX || *dp < SCHAR_MIN)
	  stat = NC_ERANGE;      
	*tp = (signed char) *dp;
      }  
      break;
  }

  return stat;
}

int
putn_into_short(void **xpp, size_t nelems, short *tp, nc_type type)
{
  signed char *bp;
  //  short *sp;
  int *ip; 
  float *fp;
  double *dp;
  int stat = NC_NOERR;

  switch (type) {
    case NC_CHAR:
      return NC_ECHAR;
    case NC_BYTE:
      bp = (signed char *) *xpp; 
      for( ; nelems != 0; nelems--, bp++, tp++)
	*tp = (short) *bp;
      break;
    case NC_SHORT:
     (void) memcpy(tp, *xpp, nelems * sizeof(short));
#ifdef Never
      sp = (short *) *xpp; 
      for( ; nelems != 0; nelems--, sp++, tp++)
	*tp = (short) *sp;
#endif
      break;
    case NC_INT:
      ip = (int *) *xpp; 
      for( ; nelems != 0; nelems--, ip++, tp++){
	if(*ip > SHORT_MAX || *ip < SHORT_MIN)
	  stat = NC_ERANGE;      
	*tp = (short) *ip;
      }  
      break;
    case NC_FLOAT: 
      fp = (float *) *xpp; 
      for( ; nelems != 0; nelems--, fp++, tp++){
	if(*fp > SHORT_MAX || *fp < SHORT_MIN)
	  stat = NC_ERANGE;      
	*tp = (short) *fp; 
      }  
      break;
    case NC_DOUBLE:
      dp = (double *) *xpp; 
      for( ; nelems != 0; nelems--, dp++, tp++){
	if(*dp > SHORT_MAX || *dp < SHORT_MIN)
	  stat = NC_ERANGE;      
	*tp = (short) *dp;
      }  
      break;
  }

  return stat;
}

int
putn_into_int(void **xpp, size_t nelems, int *tp, nc_type type)
{
  signed char *bp;
  short *sp;
  //  int *ip; 
  float *fp;
  double *dp;
  int stat = NC_NOERR;

  switch (type) {
    case NC_CHAR:
      return NC_ECHAR;
    case NC_BYTE:
      bp = (signed char *) *xpp; 
      for( ; nelems != 0; nelems--, bp++, tp++)
	*tp = (int) *bp;
      break;
    case NC_SHORT:
      sp = (short *) *xpp; 
      for( ; nelems != 0; nelems--, sp++, tp++)
	*tp = (int) *sp;
      break;
    case NC_INT:
      (void) memcpy(tp, *xpp, nelems * sizeof(int)); 
#ifdef Never
      ip = (int *) *xpp; 
      for( ; nelems != 0; nelems--, ip++, tp++)
	*tp = (int) *lp; 
#endif
      break;
    case NC_FLOAT:
      fp = (float *) *xpp; 
      for( ; nelems != 0; nelems--, fp++, tp++){
       if(*fp > (double)INT_MAX || *fp < (double)INT_MIN)
	 stat = NC_ERANGE;
       *tp = (int) *fp; 
      }  
      break;
    case NC_DOUBLE:
      dp = (double *) *xpp; 
      for( ; nelems != 0; nelems--, dp++, tp++){
	if(*dp > (double)INT_MAX || *dp < (double)INT_MIN)
	  stat = NC_ERANGE;
	*tp = (int) *dp;
      }  
      break;
  }

  return stat;
}

int
putn_into_long(void **xpp, size_t nelems, long *tp, nc_type type)
{ 
  signed char *bp;
  short *sp;
  int *ip; 
  float *fp;
  double *dp;  
  int stat = NC_NOERR;


  switch (type) {
    case NC_CHAR:
      return NC_ECHAR;
    case NC_BYTE:
      bp = (signed char *) *xpp; 
      for( ; nelems != 0; nelems--, bp++, tp++)
	*tp = (long) *bp;
      break;
    case NC_SHORT:
      sp = (short *) *xpp; 
      for( ; nelems != 0; nelems--, sp++, tp++)
	*tp = (long) *sp;
      break;
    case NC_INT:
      ip = (int *) *xpp; 
      for( ; nelems != 0; nelems--, ip++, tp++)
	*tp = (long) *ip;
      break;
    case NC_FLOAT: 
      fp = (float *) *xpp; 
      for( ; nelems != 0; nelems--, fp++, tp++){
	if(*fp > LONG_MAX || *fp < LONG_MIN)
	  stat = NC_ERANGE;
	*tp = (long) *fp; 
      }  
      break;
    case NC_DOUBLE:
      dp = (double *) *xpp; 
      for( ; nelems != 0; nelems--, dp++, tp++){
	if(*dp > LONG_MAX || *dp < LONG_MIN)
	  stat = NC_ERANGE;
	*tp = (long) *dp;
      }  
      break;
  }
  return stat;
}


int
putn_into_float(void **xpp, size_t nelems, float *tp, nc_type type)
{
  signed char *bp;
  short *sp;
  int *ip; 
  // float *fp;
  double *dp;
  int stat = NC_NOERR;

  switch (type) {
    case NC_CHAR: 
      return NC_ECHAR;
    case NC_BYTE:
      bp = (signed char *) *xpp; 
      for( ; nelems != 0; nelems--, bp++, tp++)
	*tp = (float) *bp;
      break;
    case NC_SHORT:
      sp = (short *) *xpp; 
      for( ; nelems != 0; nelems--, sp++, tp++)
	*tp = (float) *sp;
      break;
    case NC_INT:
      ip = (int *) *xpp; 
      for( ; nelems != 0; nelems--, ip++, tp++){
	//	if(*ip > FLT_MAX || *ip < (-FLT_MAX))
	//	  stat = NC_ERANGE;      
	*tp = (float) *ip;
      }  
      break;
    case NC_FLOAT: 
      (void) memcpy(tp, *xpp, nelems * sizeof(float));
#ifdef Never
      fp = (float *) *xpp; 
      for( ; nelems != 0; nelems--, fp++, tp++)
	*tp = *fp; 
#endif
      break;
    case NC_DOUBLE:
      dp = (double *) *xpp; 
      for( ; nelems != 0; nelems--, dp++, tp++){
	if(*dp > FLT_MAX || *dp < (-FLT_MAX))
	  stat= NC_ERANGE;      
	*tp = (float) *dp;
      }  
      break;
  }

  return stat;
}

int
putn_into_double(void **xpp, size_t nelems, double *tp, nc_type type)
{
  signed char *bp;
  short *sp;
  int *ip; 
  float *fp;
  // double *dp;

  switch (type) {
    case NC_CHAR:
      return NC_ECHAR;
    case NC_BYTE:
      bp = (signed char *) *xpp; 
      for( ; nelems != 0; nelems--, bp++, tp++)
	*tp = (double) *bp;
      break;
    case NC_SHORT:
      sp = (short *) *xpp; 
      for( ; nelems != 0; nelems--, sp++, tp++)
	*tp = (double) *sp;
      break;
    case NC_INT:
      ip = (int *) *xpp; 
      for( ; nelems != 0; nelems--, ip++, tp++)
	*tp = (double) *ip;
      break;
    case NC_FLOAT: 
      fp = (float *) *xpp; 
      for( ; nelems != 0; nelems--, fp++, tp++)
	*tp = (double) *fp; 
      break;
    case NC_DOUBLE:
      (void) memcpy(tp, *xpp, nelems * sizeof(double));
#ifdef Never
      dp = (double *) *xpp; 
      for( ; nelems != 0; nelems--, dp++, tp++)
	*tp =  *dp;
#endif
      break;
  }

  return NC_NOERR;
  
}

int
putn_into_text(void **xpp, size_t nelems, char *tp)
{
      (void) memcpy(tp, *xpp, nelems);
      //      *xpp = (void *)((char *)(*xpp) + nelems);
      return NC_NOERR;
}

// $Log: Dncx.cc,v $
// Revision 1.4  2000/10/06 01:22:02  jimg
// Moved the CVS Log entries to the ends of files.
// Modified the read() methods to match the new definition in the dap library.
// Added exception handlers in various places to catch exceptions thrown
// by the dap library.
//
