
// (c) COPYRIGHT URI/MIT 1994-1996
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      reza            Reza Nekovei (reza@intcomm.net)

// DODS-netCDF surrogate library for data retrieval. 
// 
// ReZa 10/14/94
//
// A rewrite to support netCDF 3.x interface. This file 
// replaces old (version 2.x) Dputget.cc and Dputgetg.cc 
// files. ReZa 5/7/99 

#include "config_nc.h"

static char rcsid[] not_used ={"$Id: Dputget.cc,v 1.5.4.1 2003/06/06 08:23:41 reza Exp $"};

#include <assert.h>

#include "Str.h"
#include "Error.h"
#include "Dnetcdf.h"
#include "Dncx.h"
#include "Dv2i.h"

//  map is the name of a template provided in VC++.  Here I
//  simple rename it out of the way to prevent the conflict
//  with the 'map' var(s) used in this source file.   If you
//  see this Reze - maybe contrive a name other than 'map'
//  and remove this ifdef.
#ifdef WIN32
#define map mapx
#endif

#define        Tvoid 0
#define        Ttext 1
#define        Tuchar 2
#define        Tschar 3
#define        Tshort 4
#define        Tint 5
#define        Tlong 6
#define        Tfloat 7
#define        Tdouble 8

extern void
var_info(int cdfid,BaseType *bt, nc_type *typep,int *ndimsp, int dims[]);

int
Dtypelen(int type) 
{
  switch(type){ // return the size of output type
        case Tvoid :
        case Ttext :
        case Tuchar :
        case Tschar :
                return((int)sizeof(char));
        case Tshort :
                return((int)sizeof(short));
        case Tint : 
                return((int)sizeof(int));
        case Tlong : 
                return((int)sizeof(long));
        case Tfloat : 
                return((int)sizeof(float));
        case Tdouble : 
                return((int)sizeof(double));
        }
        return -1;
}

// type conversion from DODS to the local machine, the
// string conversion code is now built into DODvario().
void *    
type_conv(nc_type datatype, int nels, void *tmpbufin)
{
    dods_float64 *flt64;
    dods_float32 *flt32;
    dods_int32 *intg32;
    dods_int16 *intg16;
#if 0
    string *strg;
#endif
    int id;
    void *tmpbufout = tmpbufin;

#if 0
    if (datatype == NC_CHAR) {
	char *chr = new char [nels * nctypelen(datatype)];
	strg = (string *) tmpbufin;
	tmpbufout = chr;

	for (id = 0; id < nels; id++) 
	    *chr++ = *((const char *)*(strg++));
    }
#endif

    if ((datatype == NC_FLOAT) 
	&& (sizeof(dods_float32) != lnctypelen(datatype))) {
	float *flt = (float *)new char [(nels*nctypelen(datatype))];

	flt32 = (dods_float32 *)tmpbufin;
	tmpbufout = flt;

	for (id = 0; id < nels; id++)
	    *flt++ = (float) *flt32++;
    }

    if ((datatype == NC_DOUBLE) 
	&& (sizeof(dods_float64) != lnctypelen(datatype))) {
	double *dbl = (double *)new char [(nels*nctypelen(datatype))];

	flt64 = (dods_float64 *)tmpbufin;
	tmpbufout = dbl;

	for (id = 0; id < nels; id++)
	    *dbl++ = (double) *flt64++;
    }

    if ((datatype == NC_SHORT) 
	&& (sizeof(dods_int16) != lnctypelen(datatype))) {
	short *sht = (short *)new char [(nels*nctypelen(datatype))];
	intg16 = (dods_int16 *)tmpbufin;
	tmpbufout = sht;

	for (id = 0; id < nels; id++) 
	    *sht++ = (short) *intg16++;
    }

    if ((datatype == NC_LONG) 
	&& (sizeof(dods_int32) != lnctypelen(datatype))) {
	nclong *lng = (nclong *)new char [(nels*nctypelen(datatype))];
	intg32 = (dods_int32 *)tmpbufin;
	tmpbufout = lng;

	for (id = 0; id < nels; id++) 
	    *lng++ = (nclong) *intg32++;
    }

    return tmpbufout;
}


// Function to get a hyperslap of the data into the requested buffer.
int
HyperGet(const int dim_num, int indx, const int dimsz[], nc_type typep, 
         const long *start, const long *edge, void *buff, void *value)
{
    long jump = 1;
    int szof = nctypelen(typep);

    if(dim_num != (indx+1)){
        // number of lines, pages, etc to skip 
        for (int i = dim_num-1; i > indx; i--)
            jump *= dimsz[i];

        for (int edg = 0; edg < edge[indx] ; edg++){
            void *bufft = ((char *)buff + (start[indx] + edg) * jump * szof);
            value = (char *)value + HyperGet(dim_num, indx+1, dimsz, typep, start, edge, bufft, value);
        }
        return (0);
    }
    else {
        //number of bytes to jump
        void *bufft = (char *)buff + start[indx] * szof ;
        memcpy((void *) value, (void *) bufft, (size_t)edge[indx]*szof);
        return (edge[indx] * szof);
    }
}

int
DODvario(int cdfid, int varid, const size_t *start, const size_t *edges,
	 const ptrdiff_t *stride, void *values, int outtype)
{
    static char start_rep[MAX_NC_NAME];
    static char step_rep[MAX_NC_NAME];
    static char stop_rep[MAX_NC_NAME];
    nc_type typep = NC_NAT ;
    int nels = 1;

#ifdef OldServer
    int dimsz[MAX_NC_DIMS];
    int dims[MAX_NC_DIMS];
    int ndimsp = 0 ;
#endif

    NCConnect *c = (*conns)[cdfid];

    if(( varid < 0) || (varid > c->nvars()))
      return NC_ENOTVAR;

    DDS &cdds = c->Cdds() ;
    //select the Pix for the given varid
    Pix cp = cdds.first_var();
    for (int i=0; i < varid ; i++)
	cdds.next_var(cp);

    BaseType *cbt = cdds.var(cp) ;
    string var_name = cbt->name();

    // Collect info about the variable
#ifdef OldServer
    var_info(cdfid,cbt,&typep,&ndimsp,dims);
#else
    var_info(cdfid,cbt,&typep,0,0);
#endif

    // Get dimension sizes and strings for constraint hyperslab
    string expr = var_name;
    int dm = 0;
    int ext_start, ext_step, ext_stop = 0; //external constraints
    
    // Read the Array, the Array part of the Grid
    // or a scalar value
    Array *ar = 0;
    BaseType *bt = 0;
    void *tmpbufin = 0;
    bool found_array = false;
    bool found_var = false;
    Type intype; // Atomic type of the input

    bool done = false ;
    DDS &dds = c->dds() ;
    Pix p = dds.first_var() ;
    for(; done == false && p; dds.next_var(p) )
    {
	bt = dds.var(p) ;
	if(bt->name() == var_name)
	{ 
	    switch (bt->type())
	    {
	      case dods_grid_c: {
		  Grid *gr = (Grid *)bt ; 
		  ar =  (Array *)gr->array_var();
		  intype = ar->var()->type();
		  found_var = true;
		  found_array = true;
		  break;
	      }
	      case dods_array_c: {
		  ar = (Array *)bt ; 
		  intype = ar->var()->type();
		  found_var = true;
		  found_array = true;
		  break;		
	      }
	      case dods_str_c:
	      case dods_url_c:  
	      case dods_byte_c:
	      case dods_int16_c:
	      case dods_uint16_c:
	      case dods_int32_c:
	      case dods_uint32_c: 
	      case dods_float32_c:
	      case dods_float64_c: {
		intype = bt->type();
		found_var = true;
		break;		
	      }
	      default:
#if VERBOSE_ERRORS
		cerr << "DODS: Wrong variable type." << endl;
#endif
		return (NC_EBADTYPE); // wrong type
	    }
	    done = true ;
	}
    }

    // Check for type conversions allowed 
    if(((outtype == Ttext)&&(intype != dods_str_c)&&(intype != dods_url_c)||
	((outtype != Ttext)&&(intype == dods_str_c)&&(outtype != Tvoid))||
	((outtype != Ttext)&&(intype == dods_url_c)&&(outtype != Tvoid))))
      return NC_ECHAR;

    if(!found_var) {
#if VERBOSE_ERRORS
	cerr << "DODS: Could not find variable: " << var_name << endl;
#endif
	return (NC_ENOTVAR); // no such a variable
    }

    int Zedge = 0; // Search for zero size edges (no data)
    if(found_array) {
	for (Pix d = ar->first_dim(); d; ar->next_dim(d), dm++) {
	    //Verify stride argument.
	    if (stride != NULL && stride[dm] < 1) {
		// NCadvise(NC_ESTRIDE, "Non-positive stride");
		return NC_ESTRIDE;
	    }

	    // Set defaults:
	    // *start         NULL => first corner 
	    // *edges         NULL => everything following start[] 
	    // *stride        NULL => unity strides 
	    long instart = start != NULL
		? start[dm]
		: 0;
	    long inedges = edges != NULL
		? edges[dm]
		: ar->dimension_size(d) - instart;
	    long instep = stride != NULL
		? stride[dm]
		: 1;

	    // external constarint (from ncopen)
	    ext_start = ar->dimension_start(d, true); 
	    ext_step = ar->dimension_stride(d, true);
	    ext_stop = ar->dimension_stop(d, true);

#ifdef TEST      
	    cout << instart <<" "<< inedges <<" "<< dm << endl;
	    cout << ext_start <<" "<< ext_step <<" "<<ext_stop<<endl;
#endif

	    // create constarint expr. by combining the constarints
	    int Tstart = ext_start+instart*ext_step;
	    int Tstep = instep*ext_step;
	    int Tstop = 
		(ext_stop < ext_start+(instart+(inedges-1)*instep)*ext_step) 
		? ext_stop : ext_start+(instart+(inedges-1)*instep)*ext_step;

	    // Check the validity of the array constraints
	    if((instart >= ar->dimension_size(d))||(instart < 0)||(inedges < 0))
		return NC_EINVALCOORDS;      
	    if(instart+inedges > ar->dimension_size(d))
		return NC_EEDGE;

	    // Zero edge found, but first check remaining dimensions for error
	    if(inedges == 0) 
		Zedge = 1; 

	    sprintf(start_rep,"%d",Tstart);
	    sprintf(step_rep,"%d",Tstep);	      
	    sprintf(stop_rep,"%d",Tstop);

	    expr = expr + "[" + start_rep + ":" + step_rep + ":" + stop_rep + "]";

	    nels *= inedges;
	}

	if(Zedge == 1)
	    return NC_NOERR; // Zero edge means no data

#ifdef TEST
	cout << expr <<endl;
#endif
    }    

    // ask for the sync. data with GUI on...
    DDS *Rdds = 0 ;
    try {
	Rdds = c->request_data(expr, false, false); 
	if (!Rdds) {    //NULL return, no data was sent from the server.
#if VERBOSE_ERRORS
	    cerr << "DODS: Error reading data" << endl;
#endif
	    return (-1);
	}
    }
    catch (Error &e) {
#if VERBOSE_ERRORS
	cerr << e.get_error_message() << endl;
#endif
	if(e.get_error_code() != NC_NOERR)
	  return e.get_error_code();
	else
	  return NC_EINVAL;
    }
  
    // By definition, the DDS `Rdds' can have only one variable, so make 
    // sure that is true.
    if (Rdds->num_var() != 1) { 
#if VERBOSE_ERRORS
	cerr << "DODS: Too many variables; use only single variable projections"
	     << endl;
#endif
	return (-1);
    }

    Pix qb = Rdds->first_var();  // Only one variable in this dds
    switch (Rdds->var(qb)->type()){
      case dods_grid_c: {
	  Grid *gr = (Grid *)Rdds->var(qb); 
	  ar =  (Array *)gr->array_var();
	  break;
      }
      case dods_array_c: {
	  ar = (Array *)Rdds->var(qb); 
	  break;		
      }
      case dods_str_c:
      case dods_url_c: 
      case dods_byte_c:
      case dods_int16_c:
      case dods_uint16_c:
      case dods_int32_c:
      case dods_uint32_c: 
      case dods_float32_c:
      case dods_float64_c: {
	bt = (BaseType *)Rdds->var(qb);
	break;
      }
      default: {
	//	cerr << " Wrong variable type " <<endl;
	return NC_EBADTYPE; // wrong type
      }
    }

    Type simp;
    if(found_array){
      assert(ar);		// make sure AR is not null
      simp = ar->var()->type();
    }
    else
      simp = bt->type();

    switch (simp) {
      case dods_byte_c:
      case dods_int16_c:
      case dods_uint16_c:
      case dods_int32_c:
      case dods_uint32_c: 
      case dods_float32_c:
      case dods_float64_c: {
	if(found_array)
	  rcode =  ar->buf2val((void **)&tmpbufin); 
	else
	  rcode =  bt->buf2val((void **)&tmpbufin); 
	
	if (!rcode){ // DODS retrival error
#if VERBOSE_ERRORS
	      cerr << "DODS: Unable to retrieve value" << endl;
#endif
	  delete Rdds;
	  return (-1);
	}

	  //conver to local variable types
	  char *tmpbufout =  (char *)type_conv(typep, nels, tmpbufin);
	  int szof = nctypelen(typep);

	  rcode = NC_NOERR;
	  void *xp = (void *)tmpbufout;
	  switch(outtype) {
	    case Ttext:
	      rcode = NC_ECHAR;
	      break;
	    case Tvoid:
	      memcpy((void *)values, (void *)tmpbufout, (size_t)nels*szof);
	      break;
	    case Tuchar:
	      rcode = putn_into_uchar(&xp, nels, (unsigned char *)values,
				      typep);
	      break;
	    case Tschar:
	      rcode = putn_into_schar(&xp, nels, (signed char *)values, typep);
	      break;
	    case Tshort:
	      rcode = putn_into_short(&xp, nels, (short *)values, typep);
	      break;
	    case Tint:
	      rcode = putn_into_int(&xp, nels, (int *)values, typep);
	      break;
	    case Tlong:
	      rcode = putn_into_long(&xp, nels,(long *)values, typep);
	      break;
	    case Tfloat:
	      rcode = putn_into_float(&xp, nels, (float *)values, typep);
	      break;
	    case Tdouble:
	      rcode = putn_into_double(&xp, nels, (double *)values, typep);
	      break;
	  }

	  if(tmpbufin != tmpbufout)
	      delete(tmpbufout);

	  break;
      }

      case dods_str_c:
      case dods_url_c: {
	rcode = NC_NOERR;
	char * tbfr = (char *)values;
	for (int i = 0; i < nels; i++) {
	  Str *s;
	  if(found_array)
	    s = dynamic_cast<Str *>(ar->var(i));
	  else
	    s = dynamic_cast<Str *>(bt);
	  if (!s)
	    return -1;
	  string *cp = 0;
	  string **cpp = &cp;
	  s->buf2val((void **)cpp);
 
	  for (unsigned int cntr=0; cntr<cp->length() || 
		 (cp->length()==0 && cntr==0); cntr++){
	    *tbfr = *(cp->c_str()+cntr);
	    tbfr++;
	  }
	  // Now get rid of the C++ string object.
	  delete cp;
	}
	break;
      }
	
      default:
	break;
    }

#ifdef OldServer
    // get the part asked for
    (void) HyperGet(ndimsp,0,dimsz,typep,start,edges,
		    (void *)tmpbufout,values);
#endif 

    delete Rdds;
    return rcode ;
}


int
GenRead(int cdfid, int varid, const size_t *start, const size_t *count,
	const ptrdiff_t *stride, const ptrdiff_t *imap, void *values, 
	int outtype)
{
    if (( varid < 0) || (varid > (*conns)[cdfid]->nvars()))
	return NC_ENOTVAR;

    // is memory arrangment required?
    if (imap == NULL) {
	return DODvario(cdfid, varid, start, count, stride, values, outtype);
    }
    else {
      
      //  The variable is an array with IMAP.    

      int	maxidim;	         /* maximum dimensional index */
      int	idim, ndim;            /* dim. ID and number of dim. */
      char	*valp	= (char*)values;
      int       shape[MAX_VAR_DIMS];    /* Variable shape */
      size_t	mycount[MAX_NC_DIMS];
      size_t	mystart[MAX_NC_DIMS];
      ptrdiff_t	mystride[MAX_NC_DIMS];
      ptrdiff_t	myimap[MAX_NC_DIMS];
      size_t	stop[MAX_NC_DIMS];	 /* stop indexes */
      size_t	length[MAX_NC_DIMS];	/* edge lengths in bytes */
      nc_type   datatype;              /* Variable local type */

      //select the Pix for the given varid
      Pix q = (*conns)[cdfid]->dds().first_var();

      for (int i=0; i < varid ; i++)
	(*conns)[cdfid]->dds().next_var(q);
      
      BaseType *bt = (*conns)[cdfid]->dds().var(q);

      //get the variable shape and type
      var_info(cdfid, bt, &datatype, &ndim, shape);
      maxidim = ndim - 1;

      // sizeof each value (for output type)
      int szof = Dtypelen(outtype);  

      // scalar variable with an imap?
      if (maxidim < 0) {
	// netCDF-2
	  // NCadvise(NC_EINVAL, "Scalar with index mapping");
	  //return NC_EINVAL ;

	//netCDF-3
	return DODvario(cdfid, varid, 0, 0, 0, values, outtype);//Scalar
      }

      // Verify stride argument.
      for (idim = 0; idim < ndim; ++idim) {
	if (stride != NULL && stride[idim] < 1) {
	  // NCadvise(NC_ESTRIDE, "Non-positive stride");
	  return NC_ESTRIDE ;
	}
      }
      
      // Initialize I/O parameters (set defaults).
      long nels = 1;
      for (idim = maxidim; idim >= 0; --idim) {
	mystart[idim]	= start != NULL
	  ? start[idim]
	  : 0;
	mycount[idim]	= count != NULL
	  ? count[idim]
	  : shape[idim] - mystart[idim]; 
	mystride[idim]	= stride != NULL 
	  ? stride[idim]
	  : 1;
	myimap[idim]	= imap != NULL 
	  ? imap[idim]
	  : idim == maxidim
	  ? 1
	  : myimap[idim+1] * mycount[idim+1];
	
	nels          = mycount[idim]*nels;
	length[idim]  = myimap[idim] * mycount[idim];
	stop[idim]    = mystart[idim] + mycount[idim];
      }
      
      /*
       * As an optimization, adjust I/O parameters when the fastest 
       * dimension has unity stride both externally and internally.
       * In this case, the user could have called a simpler routine
       * (i.e. ncvarget() or ncvarput()).
       */
      int iocount = 1;
      if (mystride[maxidim] == 1 && myimap[maxidim] == 1) {
	iocount             = mycount[maxidim];
	myimap[maxidim]     = length[maxidim];
      }
      
      // read only once over the net into the temp. buffer
      char *valpBuf = new char [(nels*szof)];      
      int iostat = DODvario(cdfid, varid, mystart, mycount, mystride, 
			    (void *)valpBuf,outtype);

      // For data out of range error finish IMAP first 
      if ((iostat != 0)&&(iostat != NC_ERANGE)) 
	return iostat;

      int offset = 0;
      // Perform IMAP in the buffer.  Exit when done.
      for (;;) {
	
	memcpy((void *)valp, (void *)(valpBuf+offset), (size_t)szof*iocount);
	offset += szof*iocount;	

	/*
	 * The following code permutes through the variable's external
	 * start-index space and it's internal address space.  At the 
	 * UPC, this algorithm is commonly called `odometer code'.
	 */
	idim = maxidim;
      carry:
	valp += myimap[idim]*szof;
	mystart[idim] += idim == maxidim
	  ? iocount
	  : 1;
	if (mystart[idim] >= stop[idim]) {
	  mystart[idim]	 = start[idim];
	  valp		-= length[idim]*szof;
	  if (--idim < 0){
	    delete [] valpBuf;    // remove temp buffer for imap 	    
	    return iostat;
	  }
	  goto carry;
	}
      }			  // IMAP loop 
    }	                // variable is an array with imap
}


int
ncvarputg(int cdfid, int varid, const long *start, const long *count, 
	  const long *stride, const long *imap, const void *values)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
  if((*conns)[cdfid]->is_local()) {
    rcode = lncvarputg((*conns)[cdfid]->ncid(),varid, start, count,
          stride, imap, values);
    return rcode;
  }
#endif
  cdf_routine_name = "ncvarputg" ;
  NCadvise(NC_EPERM, "DODS network interface is read-only") ;
  return NC_EPERM;
}


int
ncvarputs(int cdfid, int varid, const long *start, const long *count, 
	  const long *stride, const void *values)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
  if((*conns)[cdfid]->is_local()) {
    rcode = lncvarputs((*conns)[cdfid]->ncid(),varid, start, count,
          stride, values);
    return rcode;
  }
#endif
  cdf_routine_name = "ncvarputs" ;
  NCadvise(NC_EPERM, "DODS network interface is read-only") ;
  return NC_EPERM;
}

int
nc_get_rec(int cdfid, size_t recnum, void **datap)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_rec((*conns)[cdfid]->ncid(), recnum, datap);
	return rcode;
    }
#endif
    cdf_routine_name = "nc_get_rec" ;

    NCadvise(NC_EPERM, "Record access is not supported yet, DODS") ;
    return NC_EPERM;
}

int
ncrecget(int cdfid, long recnum, void **datap)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncrecget((*conns)[cdfid]->ncid(),recnum, datap);
	return rcode;
    }
#endif
    cdf_routine_name = "ncrecget" ;

    NCadvise(NC_EPERM, "Record access is not supported yet, DODS") ;
    return NC_EPERM;
}

int
nc_inq_rec(int cdfid, size_t *nrecvars, int *recvarids, size_t *recsizes)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode=lnc_inq_rec((*conns)[cdfid]->ncid(),nrecvars,recvarids,recsizes);
	return rcode;
    }
#endif
    cdf_routine_name = "nc_inq_rec" ;
    NCadvise(NC_EPERM, "Record access is not supported yet, DODS") ;
    return NC_EPERM;
}


int
ncrecinq(int cdfid,int *nrecvars, int *recvarids, long *recsizes)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode=lncrecinq((*conns)[cdfid]->ncid(),nrecvars,recvarids,recsizes);
	return rcode;
    }
#endif
    cdf_routine_name = "ncrecinq" ;
	
    int nrvars = -1; // <-----get number of variables with unlimited dimension
    if(nrvars == -1)
	return(-1) ;

    if(nrecvars != NULL)
	*nrecvars = nrvars;
	
    if(recvarids != NULL) {
	int ii ;
	for(ii = 0 ; ii < nrvars ; ii++) {
	    recvarids[ii] = 0; // <-get variable ID for each unlimited variable
	}
    }

    if(recsizes != NULL) {
	int ii ;
	for(ii = 0 ; ii < nrvars ; ii++) {
	    recsizes[ii] = 0; // <--get record size for each UL variable ---
	}
    }

    return nrvars ;
}

int
ncvarput1(int cdfid, int varid, const long *coords, const void *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncvarput1((*conns)[cdfid]->ncid(), varid, coords, value);
	return rcode;
    }
#endif
    cdf_routine_name = "ncvarput1" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return NC_EPERM;
}


int
ncvarput(int cdfid, int varid, const long *start, 
	 const long *edges, const void *values)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncvarput((*conns)[cdfid]->ncid(),varid,start,edges,values);
	return rcode;
    }
#endif
    cdf_routine_name = "ncvarput" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return NC_EPERM;
}

int
nc_put_rec(int cdfid, size_t recnum, void * const *datap)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_put_rec((*conns)[cdfid]->ncid(), recnum, datap);
	return rcode;
    }
#endif

    return NC_EPERM;
}

int
ncrecput(int cdfid, long recnum, void * const *datap)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncrecput((*conns)[cdfid]->ncid(), recnum, datap);
	return rcode;
    }
#endif
    cdf_routine_name = "ncrecput" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return NC_EPERM;
}

int
nctypelen(nc_type type)
{
    return lnctypelen(type);
}

int
nc_put_var1_text(int cdfid, int varid, const size_t *coord,
        const char *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var1_text((*conns)[cdfid]->ncid(), varid, coord, value);
#endif

    return NC_EPERM;
}
int
nc_put_var1_uchar(int cdfid, int varid, const size_t *coord,
        const unsigned char *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var1_uchar((*conns)[cdfid]->ncid(), varid, coord, value);
#endif

    return NC_EPERM;
}
int
nc_put_var1_schar(int cdfid, int varid, const size_t *coord,
        const signed char *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var1_schar((*conns)[cdfid]->ncid(), varid, coord, value);
#endif

    return NC_EPERM;
}

int
nc_put_var1_short(int cdfid, int varid, const size_t *coord,
		  const short *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var1_short((*conns)[cdfid]->ncid(), varid, coord, value);
#endif

    return NC_EPERM;
}
 int
nc_put_var1_int(int cdfid, int varid, const size_t *coord,
        const int *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var1_int((*conns)[cdfid]->ncid(), varid, coord, value);
#endif

    return NC_EPERM;
}
int
nc_put_var1_long(int cdfid, int varid, const size_t *coord,
        const long *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var1_long((*conns)[cdfid]->ncid(), varid, coord, value);
#endif

    return NC_EPERM;
}
 int
nc_put_var1_float(int cdfid, int varid, const size_t *coord,
        const float *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var1_float((*conns)[cdfid]->ncid(), varid, coord, value);
#endif

    return NC_EPERM;
}
 int
nc_put_var1_double(int cdfid, int varid, const size_t *coord,
        const double *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var1_double((*conns)[cdfid]->ncid(), varid, coord, value);
#endif

    return NC_EPERM;
}
 int
nc_put_var1(int cdfid, int varid, const size_t *coord, const void *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var1((*conns)[cdfid]->ncid(), varid, coord, value);
#endif

    return NC_EPERM;
}
 int
nc_put_vara_text(int cdfid, int varid,
         const size_t *start, const size_t *edges, const char *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_vara_text((*conns)[cdfid]->ncid(), varid, start,edges, value);
#endif

    return NC_EPERM;
}
 int
nc_put_vara_uchar(int cdfid, int varid,
         const size_t *start, const size_t *edges, const unsigned char *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_vara_uchar((*conns)[cdfid]->ncid(), varid, start,edges, value);
#endif

    return NC_EPERM;
}
int
nc_put_vara_schar(int cdfid, int varid,
         const size_t *start, const size_t *edges, const signed char *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_vara_schar((*conns)[cdfid]->ncid(), varid, start,edges, value);
#endif

    return NC_EPERM;
}
int
nc_put_vara_short(int cdfid, int varid,
         const size_t *start, const size_t *edges, const short *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_vara_short((*conns)[cdfid]->ncid(), varid, start,edges, value);
#endif

    return NC_EPERM;
}
int
nc_put_vara_int(int cdfid, int varid,
         const size_t *start, const size_t *edges, const int *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_vara_int((*conns)[cdfid]->ncid(), varid, start,edges, value);
#endif

    return NC_EPERM;
}
int
nc_put_vara_long(int cdfid, int varid,
         const size_t *start, const size_t *edges, const long *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_vara_long((*conns)[cdfid]->ncid(), varid, start,edges, value);
#endif

    return NC_EPERM;
}
 
int
nc_put_vara_float(int cdfid, int varid,
         const size_t *start, const size_t *edges, const float *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_vara_float((*conns)[cdfid]->ncid(), varid, start,edges, value);
#endif

    return NC_EPERM;
}
int
nc_put_vara_double(int cdfid, int varid,
         const size_t *start, const size_t *edges, const double *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_vara_double((*conns)[cdfid]->ncid(), varid, start,edges, value);
#endif

    return NC_EPERM;
}
 int
nc_put_vara(int cdfid, int varid,
         const size_t *start, const size_t *edges, const void *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_vara((*conns)[cdfid]->ncid(), varid, start,edges, value);
#endif

    return NC_EPERM;
}
int
nc_put_var_text(int cdfid, int varid, const char *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var_text((*conns)[cdfid]->ncid(), varid, value);
#endif

    return NC_EPERM;
}
int
nc_put_var_uchar(int cdfid, int varid, const unsigned char *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var_uchar((*conns)[cdfid]->ncid(), varid, value);
#endif

    return NC_EPERM;
}
int
nc_put_var_schar(int cdfid, int varid, const signed char *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var_schar((*conns)[cdfid]->ncid(), varid, value);
#endif

    return NC_EPERM;
}
int
nc_put_var_short(int cdfid, int varid, const short *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var_short((*conns)[cdfid]->ncid(), varid, value);
#endif

    return NC_EPERM;
}
int
nc_put_var_int(int cdfid, int varid, const int *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var_int((*conns)[cdfid]->ncid(), varid, value);
#endif

    return NC_EPERM;
}
int
nc_put_var_long(int cdfid, int varid, const long *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var_long((*conns)[cdfid]->ncid(), varid, value);
#endif

    return NC_EPERM;
}
int
nc_put_var_float(int cdfid, int varid, const float *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var_float((*conns)[cdfid]->ncid(), varid, value);
#endif

    return NC_EPERM;
}
int
nc_put_var_double(int cdfid, int varid, const double *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_var_double((*conns)[cdfid]->ncid(), varid, value);
#endif

    return NC_EPERM;
}

int
nc_put_varm_text(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride, const ptrdiff_t *map,
        const char *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_varm_text((*conns)[cdfid]->ncid(), varid,
        start, edges, stride, map, value);
#endif

    return NC_EPERM;
}
int
nc_put_varm_uchar(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride, const ptrdiff_t *map,
        const unsigned char *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_varm_uchar((*conns)[cdfid]->ncid(), varid,
        start, edges, stride, map, value);
#endif

    return NC_EPERM;
}
int
nc_put_varm_schar(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride, const ptrdiff_t *map,
        const signed char *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_varm_schar((*conns)[cdfid]->ncid(), varid,
        start, edges, stride, map, value);
#endif

    return NC_EPERM;
} 
int
nc_put_varm_short(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride, const ptrdiff_t *map,
        const short *value)
{ 
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_varm_short((*conns)[cdfid]->ncid(), varid,
        start, edges, stride, map, value);
#endif

    return NC_EPERM;
}
int
nc_put_varm_int(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride, const ptrdiff_t *map,
        const int *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_varm_int((*conns)[cdfid]->ncid(), varid,
        start, edges, stride, map, value);
#endif

    return NC_EPERM;
}
int
nc_put_varm_long(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride, const ptrdiff_t *map,
        const long *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_varm_long((*conns)[cdfid]->ncid(), varid,
        start, edges, stride, map, value);
#endif

    return NC_EPERM;
}
int
nc_put_varm_float(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride, const ptrdiff_t *map,
        const float *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_varm_float((*conns)[cdfid]->ncid(), varid,
        start, edges, stride, map, value);
#endif

    return NC_EPERM;
}

int
nc_put_varm_double(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride, const ptrdiff_t *map,
        const double *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_varm_double((*conns)[cdfid]->ncid(), varid,
        start, edges, stride, map, value);
#endif

    return NC_EPERM;
}

int
nc_put_varm (
        int cdfid, int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        const ptrdiff_t * map,
        const void *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_put_varm((*conns)[cdfid]->ncid(), varid,
        start, edges, stride, map, value);
#endif
    return NC_EPERM;
}
 
int
nc_put_vars_text (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        const char *value)
{
        return nc_put_varm_text (ncid, varid, start, edges,
                         stride, 0, value);
}

int
nc_put_vars_uchar (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        const unsigned char *value)
{
        return nc_put_varm_uchar (ncid, varid, start, edges,
                         stride, 0, value);
}

int
nc_put_vars_schar (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        const signed char *value)
{
        return nc_put_varm_schar (ncid, varid, start, edges,
                         stride, 0, value);
}

int
nc_put_vars_short (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        const short *value)
{
        return nc_put_varm_short (ncid, varid, start, edges,
                         stride, 0, value);
}
int
nc_put_vars_int (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        const int *value)
{
        return nc_put_varm_int (ncid, varid, start, edges,
                         stride, 0, value);
}

int
nc_put_vars_long (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        const long *value)
{
        return nc_put_varm_long (ncid, varid, start, edges,
                         stride, 0, value);
}

int
nc_put_vars_float (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        const float *value)
{
        return nc_put_varm_float (ncid, varid, start, edges,
                         stride, 0, value);
}

int
nc_put_vars_double (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        const double *value)
{
        return nc_put_varm_double (ncid, varid, start, edges,
                         stride, 0, value);
}
int
nc_put_vars (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        const void *value)
{
        return nc_put_varm (ncid, varid, start, edges,
                         stride, 0, value);
}


int
nc_get_var1_text(int cdfid, int varid, const size_t *coord, char *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_var1_text((*conns)[cdfid]->ncid(), varid, coord, value);
	return rcode;
    }
#endif

  NDIMS_DECL
  A_DECL(count, size_t, ndims, index);

  for (unsigned int i = 0; i < ndims ; i++)
    count[i] = 1;     // set count to one
  
  rcode= DODvario(cdfid, varid, coord, count, 0, value, Ttext);

  A_FREE(count);
  return rcode;
}
 
int
nc_get_var1_uchar(int cdfid, int varid, const size_t *coord, unsigned char *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_var1_uchar((*conns)[cdfid]->ncid(), varid, coord, value);
	return rcode;
    }
#endif

  NDIMS_DECL
  A_DECL(count, size_t, ndims, index);

  for (unsigned int i =0; i < ndims ; i++)
    count[i] = 1;     // set count to one
  
  rcode= DODvario(cdfid, varid, coord, count, 0, value, Tuchar);

  A_FREE(count);
  return rcode;
}

int
nc_get_var1_schar(int cdfid, int varid, const size_t *coord, signed char *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_var1_schar((*conns)[cdfid]->ncid(), varid, coord, value);
	return rcode;
    }
#endif

  NDIMS_DECL
  A_DECL(count, size_t, ndims, index);

  for (unsigned int i =0; i < ndims ; i++)
    count[i] = 1;     // set count to one
  
  rcode= DODvario(cdfid, varid, coord, count, 0, value, Tschar);

  A_FREE(count);
  return rcode;
}
 
int
nc_get_var1_short(int cdfid, int varid, const size_t *coord, short *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_var1_short((*conns)[cdfid]->ncid(), varid, coord, value);
	return rcode;
    }
#endif

  NDIMS_DECL
  A_DECL(count, size_t, ndims, index);

  for (unsigned int i =0; i < ndims ; i++)
    count[i] = 1;     // set count to one
  
  rcode= DODvario(cdfid, varid, coord, count, 0, value, Tshort);

  A_FREE(count);
  return rcode;
}
 
int
nc_get_var1_int(int cdfid, int varid, const size_t *coord, int *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_var1_int((*conns)[cdfid]->ncid(), varid, coord, value);
	return rcode;
    }
#endif

  NDIMS_DECL
  A_DECL(count, size_t, ndims, index);

  for (unsigned int i =0; i < ndims ; i++)
    count[i] = 1;     // set count to one
  
  rcode= DODvario(cdfid, varid, coord, count, 0, value, Tint);

  A_FREE(count);
  return rcode;
}
 
int
nc_get_var1_long(int cdfid, int varid, const size_t *coord, long *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_var1_long((*conns)[cdfid]->ncid(), varid, coord, value);
	return rcode;
    }
#endif

  NDIMS_DECL
  A_DECL(count, size_t, ndims, index);

  for (unsigned int i =0; i < ndims ; i++)
    count[i] = 1;     // set count to one
  
  rcode= DODvario(cdfid, varid, coord, count, 0, value, Tlong);

  A_FREE(count);
  return rcode;
}
 
int
nc_get_var1_float(int cdfid, int varid, const size_t *coord, float *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_var1_float((*conns)[cdfid]->ncid(), varid, coord, value);
	return rcode;
    }
#endif

  NDIMS_DECL
  A_DECL(count, size_t, ndims, index);

  for (unsigned int i =0; i < ndims ; i++)
    count[i] = 1;     // set count to one
  
  rcode= DODvario(cdfid, varid, coord, count, 0, value, Tfloat);

  A_FREE(count);
  return rcode;
}

int
nc_get_var1_double(int cdfid, int varid, const size_t *coord, double *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_var1_double((*conns)[cdfid]->ncid(), varid, coord, value);
	return rcode;
    }
#endif

  NDIMS_DECL
  A_DECL(count, size_t, ndims, index);

  for (unsigned int i =0; i < ndims ; i++)
    count[i] = 1;     // set count to one
  
  rcode= DODvario(cdfid, varid, coord, count, 0, value, Tdouble);

  A_FREE(count);
  return rcode;
}

int
nc_get_var1(int cdfid, int varid, const size_t *coord, void *value)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lnc_get_var1((*conns)[cdfid]->ncid(), varid, coord, value);
	return rcode;
    }
#endif

  NDIMS_DECL
  A_DECL(count, size_t, ndims, index);

  for (unsigned int i =0; i < ndims ; i++)
    count[i] = 1;     // set count to one
  
  rcode= DODvario(cdfid, varid, coord, count, 0, value, Tvoid);

  A_FREE(count);
  return rcode;
}
 

int
nc_get_vara_text(int cdfid, int varid,
         const size_t *start, const size_t *edges, char *value)
{    
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_vara_text((*conns)[cdfid]->ncid(), varid,
			       start, edges, value);
#endif

    return DODvario(cdfid, varid, start, edges, 0, value, Ttext);
}
int
nc_get_vara_uchar(int cdfid, int varid,
         const size_t *start, const size_t *edges, unsigned char *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_vara_uchar((*conns)[cdfid]->ncid(), varid,
			       start, edges, value);
#endif

    return DODvario(cdfid, varid, start, edges, 0, value, Tuchar);
}
int
nc_get_vara_schar(int cdfid, int varid,
         const size_t *start, const size_t *edges, signed char *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_vara_schar((*conns)[cdfid]->ncid(), varid,
			       start, edges, value);
#endif

    return DODvario(cdfid, varid, start, edges, 0, value, Tschar);
}
int
nc_get_vara_short(int cdfid, int varid,
         const size_t *start, const size_t *edges, short *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_vara_short((*conns)[cdfid]->ncid(), varid,
			       start, edges, value);
#endif

    return DODvario(cdfid, varid, start, edges, 0, value, Tshort);
}
int
nc_get_vara_int(int cdfid, int varid,
         const size_t *start, const size_t *edges, int *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_vara_int((*conns)[cdfid]->ncid(), varid,
			       start, edges, value);
#endif

    return DODvario(cdfid, varid, start, edges, 0, value, Tint);
}
int
nc_get_vara_long(int cdfid, int varid,
         const size_t *start, const size_t *edges, long *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_vara_long((*conns)[cdfid]->ncid(), varid,
			       start, edges, value);
#endif

    return DODvario(cdfid, varid, start, edges, 0, value, Tlong);
}
int
nc_get_vara_float(int cdfid, int varid,
         const size_t *start, const size_t *edges, float *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_vara_float((*conns)[cdfid]->ncid(), varid,
			       start, edges, value);
#endif

    return DODvario(cdfid, varid, start, edges, 0, value, Tfloat);
}
 
int
nc_get_vara_double(int cdfid, int varid,
         const size_t *start, const size_t *edges, double *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_vara_double((*conns)[cdfid]->ncid(), varid,
			       start, edges, value);
#endif

    return DODvario(cdfid, varid, start, edges, 0, value, Tdouble);
}
int
nc_get_vara(int cdfid, int varid,
         const size_t *start, const size_t *edges, void *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    NCConnect *c = (*conns)[cdfid];
    if(c->is_local()) 
	return lnc_get_vara(c->ncid(), varid, start, edges, value);
#endif

    return DODvario(cdfid, varid, start, edges, 0, value, Tvoid);
}

int
nc_get_var_text(int cdfid, int varid, char *value)
{  
  if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
      || (*conns)[cdfid] == NULL) // does the file id exist?
    return NC_EBADID;

#ifdef LOCAL
  if((*conns)[cdfid]->is_local()) 
    return lnc_get_var_text((*conns)[cdfid]->ncid(), varid, value);
#endif

  return DODvario(cdfid, varid, 0, 0, 0, value, Ttext);
}

int
nc_get_var_uchar(int cdfid, int varid, unsigned char *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_var_uchar((*conns)[cdfid]->ncid(), varid, value);
#endif

    return DODvario(cdfid, varid, 0, 0, 0, value, Tuchar);
}
int
nc_get_var_schar(int cdfid, int varid, signed char *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_var_schar((*conns)[cdfid]->ncid(), varid, value);
#endif

    return DODvario(cdfid, varid, 0, 0, 0, value, Tschar);
}
int
nc_get_var_short(int cdfid, int varid, short *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_var_short((*conns)[cdfid]->ncid(), varid, value);
#endif

    return DODvario(cdfid, varid, 0, 0, 0, value, Tshort);
}
int
nc_get_var_int(int cdfid, int varid, int *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_var_int((*conns)[cdfid]->ncid(), varid, value);
#endif

    return DODvario(cdfid, varid, 0, 0, 0, value, Tint);
}
int
nc_get_var_long(int cdfid, int varid, long *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_var_long((*conns)[cdfid]->ncid(), varid, value);
#endif

    return DODvario(cdfid, varid, 0, 0, 0, value, Tlong);
}
int
nc_get_var_float(int cdfid, int varid, float *value)
{  
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_var_float((*conns)[cdfid]->ncid(), varid, value);
#endif

    return DODvario(cdfid, varid, 0, 0, 0, value, Tfloat);
}
int
nc_get_var_double(int cdfid, int varid, double *value)
{   
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_var_double((*conns)[cdfid]->ncid(), varid, value);
#endif

    return DODvario(cdfid, varid, 0, 0, 0, value, Tdouble);
} 

int
nc_get_varm_text(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride,
        const ptrdiff_t *map,
        char *value)
{   
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_varm_text((*conns)[cdfid]->ncid(), varid, start,
			       edges, stride, map, value);
#endif

 return GenRead(cdfid, varid, start, edges, 
		 stride, map, (void *)value, Ttext);
}
int
nc_get_varm_uchar(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride,
        const ptrdiff_t *map,
        unsigned char *value)
{   
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_varm_uchar((*conns)[cdfid]->ncid(), varid, start,
			       edges, stride, map, value);
#endif

 return GenRead(cdfid, varid, start, edges, 
		 stride, map, (void *)value, Tuchar);
}
int
nc_get_varm_schar(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride,
        const ptrdiff_t *map,
        signed char *value)
{   
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_varm_schar((*conns)[cdfid]->ncid(), varid, start,
			       edges, stride, map, value);
#endif

 return GenRead(cdfid, varid, start, edges, 
		 stride, map, (void *)value, Tschar);
}
 int
nc_get_varm_short(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride,
        const ptrdiff_t *map,
        short *value)
{   
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_varm_short((*conns)[cdfid]->ncid(), varid, start,
			       edges, stride, map, value);
#endif

 return GenRead(cdfid, varid, start, edges, 
		 stride, map, (void *)value, Tshort);
} 
int
nc_get_varm_int(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride,
        const ptrdiff_t *map,
        int *value)
{   
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_varm_int((*conns)[cdfid]->ncid(), varid, start,
			       edges, stride, map, value);
#endif

 return GenRead(cdfid, varid, start, edges, 
		 stride, map, (void *)value, Tint);
} 
int
nc_get_varm_long(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride,
        const ptrdiff_t *map,
        long *value)
{   
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_varm_long((*conns)[cdfid]->ncid(), varid, start,
			       edges, stride, map, value);
#endif

 return GenRead(cdfid, varid, start, edges, 
		 stride, map, (void *)value, Tlong);
} 
int
nc_get_varm_float(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride,
        const ptrdiff_t *map,
        float *value)
{   
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_varm_float((*conns)[cdfid]->ncid(), varid, start,
			       edges, stride, map, value);
#endif

 return GenRead(cdfid, varid, start, edges, 
		 stride, map, (void *)value, Tfloat);
}
int
nc_get_varm_double(int cdfid, int varid,
        const size_t *start, const size_t *edges,
        const ptrdiff_t *stride,
        const ptrdiff_t *map,
        double *value)
{   
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_varm_double((*conns)[cdfid]->ncid(), varid, start,
			       edges, stride, map, value);
#endif

 return GenRead(cdfid, varid, start, edges, 
		 stride, map, (void *)value, Tdouble);
}
int
nc_get_varm (
        int cdfid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        const ptrdiff_t * map,
        void *value)
{   
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) 
      return lnc_get_varm((*conns)[cdfid]->ncid(), varid, start,
			       edges, stride, map, value);
#endif

 return GenRead(cdfid, varid, start, edges, 
		 stride, map, (void *)value, Tvoid);
}
int
nc_get_vars_text (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        char *value)
{
        return nc_get_varm_text (ncid, varid, start, edges,
                         stride, 0, value);
}


int
nc_get_vars_uchar (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        unsigned char *value)
{
        return nc_get_varm_uchar (ncid, varid, start, edges,
                         stride, 0, value);
}

int
nc_get_vars_schar (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        signed char *value)
{
        return nc_get_varm_schar (ncid, varid, start, edges,
                         stride, 0, value);
}

int
nc_get_vars_short (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        short *value)
{
        return nc_get_varm_short (ncid, varid, start, edges,
                         stride, 0, value);
}
int
nc_get_vars_int (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        int *value)
{
        return nc_get_varm_int (ncid, varid, start, edges,
                         stride, 0, value);
}

int
nc_get_vars_long (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        long *value)
{
        return nc_get_varm_long (ncid, varid, start, edges,
                         stride, 0, value);
}

int
nc_get_vars_float (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        float *value)
{
        return nc_get_varm_float (ncid, varid, start, edges,
                         stride, 0, value);
}

int
nc_get_vars_double (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        double *value)
{
        return nc_get_varm_double (ncid, varid, start, edges,
                         stride, 0, value);
}


int
nc_get_vars (
        int ncid,
        int varid,
        const size_t * start,
        const size_t * edges,
        const ptrdiff_t * stride,
        void *value)
{
        return nc_get_varm (ncid, varid, start, edges,
                         stride, 0, value);
}

// begin version 2 interface
int
ncvarget1(
    int         cdfid,
    int         varid,
    const long* index,
    void*       value)
{
        NDIMS_DECL
        A_DECL(coordp, size_t, ndims, index);
        A_INIT(coordp, size_t, ndims, index);
        {
        const int status = nc_get_var1(cdfid, varid, coordp, value);
        A_FREE(coordp);
        if(status != NC_NOERR)
        {
	  cdf_routine_name = "ncvarget1" ;
	  NCadvise(status, "Error in getting the data") ;
	  return status;
        }
        }
        return 0;
}
int
ncvarget(
    int         cdfid,
    int         varid,
    const long* start,
    const long* count, 
    void*       value)
{
        NDIMS_DECL
        A_DECL(stp, size_t, ndims, start);
        A_DECL(cntp, size_t, ndims, count);
        A_INIT(stp, size_t, ndims, start);
        A_INIT(cntp, size_t, ndims, count);
        {
        const int status = nc_get_vara(cdfid, varid, stp, cntp, value);
        A_FREE(cntp);
        A_FREE(stp);
        if(status != NC_NOERR)
        {
	  cdf_routine_name = "ncvarget" ;
	  NCadvise(status, "Error in getting the data") ;
	  return status;
        }
        }
        return 0;
}

int
ncvargets(
    int         cdfid,
    int         varid,
    const long* start,
    const long* count,
    const long* stride,
    void*       value)
{
        if(stride == NULL)
                return ncvarget(cdfid, varid, start, count, value);
        /* else */
        {
        NDIMS_DECL
        A_DECL(stp, size_t, ndims, start);
        A_DECL(cntp, size_t, ndims, count);
        A_DECL(strdp, ptrdiff_t, ndims, stride);
        A_INIT(stp, size_t, ndims, start);
        A_INIT(cntp, size_t, ndims, count);
        A_INIT(strdp, ptrdiff_t, ndims, stride);
        {
        const int status = nc_get_vars(cdfid, varid, stp, cntp, strdp, value);
        A_FREE(strdp);
        A_FREE(cntp);
        A_FREE(stp);
        if(status != NC_NOERR)
        {
	  cdf_routine_name = "ncvargets" ;
	  NCadvise(status, "Error in getting the data") ;
	  return status;
        }
        }
        return 0;
        }
}
int
ncvargetg(
    int         cdfid,
    int         varid,
    const long* start,
    const long* count,
    const long* stride,
    const long* map,
    void*       value
)
{
        if(map == NULL)
                return ncvargets(cdfid, varid, start, count, stride, value);
        /* else */
        {
        NDIMS_DECL
        A_DECL(stp, size_t, ndims, start);
        A_DECL(cntp, size_t, ndims, count);
        A_DECL(strdp, ptrdiff_t, ndims, stride);
        A_DECL(imp, ptrdiff_t, ndims, map);
        A_INIT(stp, size_t, ndims, start);
        A_INIT(cntp, size_t, ndims, count);
        A_INIT(strdp, ptrdiff_t, ndims, stride);
        A_INIT(imp, ptrdiff_t, ndims, map);
        {
        const int status = nc_get_varm(cdfid, varid,
                        stp, cntp, strdp, imp, value);
        A_FREE(imp);
        A_FREE(strdp);
        A_FREE(cntp);
        A_FREE(stp);
        if(status != NC_NOERR)
        {
	  cdf_routine_name = "ncvargetg" ;
	  NCadvise(status, "Error in getting the data") ;
	  return status;
        }
        }
        return 0;
        }
}

// $Log: Dputget.cc,v $
// Revision 1.5.4.1  2003/06/06 08:23:41  reza
// Updated the servers to netCDF-3 and fixed error handling between client and server.
//
// Revision 1.5  2003/01/28 07:08:24  jimg
// Merged with release-3-2-8.
//
// Revision 1.4.4.4  2002/12/27 00:36:34  jimg
// Made serveral loop variables unsigned to quiet warnings about mixed compares.
//
// Revision 1.4.4.3  2002/12/05 20:32:39  pwest
// Corrected problems with IteratorAdapter code and cleaned up file descriptors
// and memory management problems.
//
// Revision 1.4.4.2  2002/11/21 23:02:19  pwest
// fixed some memory leaks and memory read errors
//
// Revision 1.4.4.1  2002/11/18 01:11:37  rmorris
// VC++ required an explicit cast to Pix in certain cases.
//
// Revision 1.4  2000/10/06 01:22:02  jimg
// Moved the CVS Log entries to the ends of files.
// Modified the read() methods to match the new definition in the dap library.
// Added exception handlers in various places to catch exceptions thrown
// by the dap library.
//
// Revision 1.3  1999/11/05 05:15:04  jimg
// Result of merge woth 3-1-0
//
// Revision 1.1.2.3  1999/10/31 16:09:21  reza
// Removed incompatibilities in error handling and added support for scalar types.
//
// Revision 1.1.2.2  1999/10/29 05:05:19  jimg
// Reza's fixes plus the configure & Makefile update
//
// Revision 1.2  1999/10/21 13:19:06  reza
// IMAP and other bugs fixed for version3.
//
// Revision 1.1.2.1  1999/10/15 19:50:56  jimg
// Changed return values and conditions for NC API entry points
//
// Revision 1.1  1999/07/28 00:22:41  jimg
// Added
//
// Revision 1.21.2.2  1999/05/27 18:35:33  reza
// Added a fix for null strings.
//
// Revision 1.21.2.1  1999/05/27 17:43:22  reza
// Fixed bugs in string changes
//
// Revision 1.21  1999/05/07 23:45:31  jimg
// String --> string fixes
//
// Revision 1.20  1999/04/01 20:42:03  jimg
// Fixed a bug in the handling of arrays of String variables.
//
// Revision 1.19  1999/03/30 05:20:55  reza
// Added support for the new data types (Int16, UInt16, and Float32).
//
// Revision 1.18  1998/02/05 20:14:44  jimg
// DODS now compiles with gcc 2.8.x
//
// Revision 1.17  1997/03/04 17:18:18  jimg
// Added `delete Rdds' to DODvario so that DDS objects allocated on the heap
// are freed once they are no longer needed.
// Changed the call `free tmpbufout' to `delete tmpbufout' since tmpbufout is
// allocated using new (see type_conv).
//
// Revision 1.16  1997/02/12 06:37:47  reza
// Changed DDS returned by request_data to a pointer.
// Add check for Null DDS and turned the GUI on.
//
// Revision 1.15  1996/09/17 17:06:09  jimg
// Merge the release-2-0 tagged files (which were off on a branch) back into
// the trunk revision.
//
// Revision 1.14.2.4  1996/09/17 00:26:10  jimg
// Merged changes from a side branch which contained various changes from
// Reza and Charles.
// Removed ncdump and netexec since ncdump is now in its own directory and
// netexec is no longer used.
//
// Revision 1.14.2.3  1996/08/13 21:18:57  jimg
// *** empty log message ***
//
// Revision 1.14.2.2  1996/07/10 21:43:49  jimg
// Changes for version 2.06. These fixed lingering problems from the migration
// from version 1.x to version 2.x.
// Removed some (but not all) warning generated with gcc's -Wall option.
//
// Revision 1.14.2.1  1996/06/25 22:04:12  jimg
// Version 2.0 from Reza.
//
// Revision 1.14  1995/11/18  11:33:22  reza
// Updated member function names for DAP-1.1.1.
//
// Revision 1.13  1995/07/09  21:33:34  jimg
// Added copyright notice.
//
// Revision 1.12  1995/06/28  20:22:36  jimg
// Replaced malloc calls with calls to new (and calls to free with calls to
// delete).
//
// Revision 1.11  1995/06/23  20:14:08  reza
// Updated DODvario() call.
//
// Revision 1.10  1995/06/23  19:53:32  jimg
// Added break.
// Reformatted.
//
// Revision 1.9  1995/06/23  15:27:00  jimg
// Changed DODvario() so that it makes use of the new netio Connect class
// methods.
//
// Revision 1.8  1995/04/28  20:39:57  reza
// Moved hyperslab access to the server side (passed as a constraint argument).
//
// Revision 1.7  1995/03/23  15:03:09  reza
// Fixed a bug in HyperGet recursion.
//
// Revision 1.6  1995/03/16  16:56:24  reza
// Updated for the new DAP. All the read_val mfunc. and their memory management
// are now moved to their parent class.
// Data transfers are now in binary while DAS and DDS are still sent in ASCII.
//
// Revision 1.5  1995/02/10  04:51:58  reza
// Finished data transmiton code and added Hyperslab and type conversion func.
//
// Revision 1.4  1994/12/22  04:45:34  reza
// Rearranged for simpler DODS data transmission code interface.
//
// Revision 1.3  1994/12/08  19:01:30  reza
// Added support for local netCDF file access through the original netCDF code.
//
// Revision 1.2  1994/11/23  21:06:00  reza
// First working version.
//
// Revision 1.1  1994/11/03  04:35:47  reza
// Preliminary version of netCDF -> DODS library.
