
// (c) COPYRIGHT URI/MIT 1994-1996
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      reza            Reza Nekovei (reza@intcomm.net)

// DODS-netCDF surrogate library for data variables. 
// 
// ReZa 10/6/94

#include "config_nc.h"

static char rcsid[] not_used ={"$Id: Dvar.cc,v 1.5 2003/01/28 07:08:24 jimg Exp $"};

#include "Dnetcdf.h"
#include "BaseType.h"
#include "Array.h"
#include "Grid.h"

int
nc_def_var( int cdfid, const char *name, nc_type type,
         int ndims, const int *dimids, int *varidp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode=lnc_def_var((*conns)[cdfid]->ncid(), name, type, ndims, dimids, varidp);
      return rcode;
    }
#endif

    return NC_EPERM;
}


int
ncvardef(int cdfid, const char *name, nc_type type, int ndims, 
	 const int dims[]) // Support version 2 interface
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lncvardef((*conns)[cdfid]->ncid(), name, type, ndims, dims);
	return rcode;
    }
#endif

    cdf_routine_name = "ncvardef" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}

int
nc_inq_varid(int cdfid, const char *name, int *varid_ptr)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode=lnc_inq_varid((*conns)[cdfid]->ncid(), name, varid_ptr);
      return rcode;
    }
#endif

    int varid = -1;
    int i = 0;
    for (Pix p = (*conns)[cdfid]->Cdds().first_var(); 
	 p;  
	 i++, (*conns)[cdfid]->Cdds().next_var(p)) {
	if(strcmp((*conns)[cdfid]->Cdds().var(p)->name().c_str(), name) == 0) {
	    varid=i;  
	    break;
	}
    }

    if (varid == -1)
      return NC_ENOTVAR;
    
    *varid_ptr = varid;
    return NC_NOERR;
}


int
ncvarid(int cdfid, const char *name) // Support version 2 interface
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lncvarid( (*conns)[cdfid]->ncid(), name);
	return rcode;
    }
#endif

    cdf_routine_name = "ncvarid" ;

    rcode = -1;
    int i = 0;
    for (Pix p = (*conns)[cdfid]->Cdds().first_var(); 
	 p;  
	 i++, (*conns)[cdfid]->Cdds().next_var(p)) {
	if(strcmp((*conns)[cdfid]->Cdds().var(p)->name().c_str(), name) == 0) {
	    rcode=i;  
	    break;
	}
    }

    if (rcode == -1)     
	NCadvise(NC_ENOTVAR, "variable \"%s\" not found", name) ;
  
    return rcode ;
}


int
nc_rename_var(int cdfid, int varid, const char *newname)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode=lnc_rename_var((*conns)[cdfid]->ncid(), varid, newname);
      return rcode;
    }
#endif

    return NC_EPERM;
}


int
ncvarrename(int cdfid, int varid, const char *newname)  
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncvarrename((*conns)[cdfid]->ncid(), varid, newname);
	return rcode;
    }
#endif
    cdf_routine_name = "ncvarrename" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}	


// Currently only handles BaseTypes, Grid and Array

void
var_info(int cdfid,BaseType *bt, nc_type *typep,int *ndimsp, int dims[])
{
    Array *ar = NULL ;
    Grid *gr = NULL ;
    int ii = 0;

    Type vartype = bt->type();
    //    string vartype = bt->type_name();
    //set default values for single value base types (non-array)
    if(dims != 0)
	dims[0] = 1;
    if(ndimsp != 0)
	*ndimsp = 0;

    for(;;) {
	if(vartype == dods_int32_c){
	    if(typep != 0)
		*typep =  NC_LONG;
	    break;
	}

	if(vartype == dods_uint32_c ){
	    if(typep != 0)
	        *typep =  NC_LONG;
	   break;
	}

	if(vartype == dods_int16_c){
	    if(typep != 0)
		*typep =  NC_SHORT;
	    break;
	}

	if(vartype == dods_uint16_c){
	    if(typep != 0)
		*typep =  NC_SHORT;
	    break;
	}

	if(vartype == dods_float64_c){
	    if(typep != 0)
		*typep =  NC_DOUBLE;
	    break;
	}

	if(vartype == dods_float32_c){
	    if(typep != 0)
		*typep =  NC_FLOAT;
	    break;
	}
	
	if(vartype == dods_str_c){
	    if(typep != 0)
		*typep =  NC_CHAR;
	    break;
	}

	if(vartype == dods_byte_c){
	    if(typep != 0)
		*typep =  NC_BYTE;
	    break;
	}

	if(vartype == dods_grid_c){
	    gr =  (Grid *)bt;
	    ar =  (Array *)gr->array_var();
	    vartype =  ar->type();
	    bt = ar;
	}

	if(vartype == dods_array_c ){
	    ar =  (Array *)bt;

	    // set the number of dimentions and the shape for this array
	    if(dims != 0) {
		for (Pix d = ar->first_dim(); d; ii++, ar->next_dim(d)) {

		    // get dim. IDs for shape construction
		    // match using dimension's name and size, 
		    // if not available use dim. size only
		    for (int jj=0; jj < (*conns)[cdfid]->ndims() ; jj++) {
			if (ar->dimension_name(d) == ""){

			    if ((*conns)[cdfid]->dim_size(jj) 
				== ar->dimension_size(d))
				dims[ii] = jj;
			} else {

			    if (((*conns)[cdfid]->dim_size(jj) 
				 == ar->dimension_size(d))
				&& ((*conns)[cdfid]->dim_name(jj) == ar->dimension_name(d)))
				dims[ii] = jj;       
			}
		    }
		}
	    }
     
	    if(ndimsp != 0)
		*ndimsp = ar->dimensions();

	    ar = (Array *)ar->var();
	    vartype = ar->type();
	    bt = ar;
	}
    }
}

int
num_attr(int cdfid, int varid)
{
    extern Pix  get_das_table(int cdfid,int varid);

    Pix q = get_das_table(cdfid,varid); // get das table

    NCConnect *c = (*conns)[cdfid];
    DAS &das = c->das() ;
    // search the attribute table 
    Pix p = das.get_table(q)->first_attr();
    int i;
    for (i = 0; p ; i++)
	das.get_table(q)->next_attr(p);

    return i;
}

int
nc_inq_var(int cdfid, int varid, char *name, nc_type *typep,
        int *ndimsp, int *dimids, int *nattsp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode= lnc_inq_var((*conns)[cdfid]->ncid(), varid, name, typep,
			 ndimsp, dimids, nattsp);
      return rcode;
    }
#endif

  // varid = -1 is used for global atrributes
    if(( varid < 0) || (varid > (*conns)[cdfid]->nvars()))
      return NC_ENOTVAR;

    //select the Pix for the given varid
    Pix q = (*conns)[cdfid]->Cdds().first_var();
    for (int i=0; i < varid ; i++)
	(*conns)[cdfid]->Cdds().next_var(q);

    BaseType *bt = (*conns)[cdfid]->Cdds().var(q);

    if(name != NULL)
	(void) strcpy(name, bt->name().c_str());

    if((typep != 0)||(ndimsp != 0)||(dimids != 0))
	var_info(cdfid,bt,typep,ndimsp,dimids);

    if(nattsp != 0)
	*nattsp = num_attr(cdfid,varid);
    
    return NC_NOERR;
}


int 
nc_inq_varname(int cdfid, int varid, char *name)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode= lnc_inq_varname((*conns)[cdfid]->ncid(), varid, name);
      return rcode;
    }
#endif

    if(( varid < 0) || (varid > (*conns)[cdfid]->nvars()))
      return NC_ENOTVAR;

    //select the Pix for the given varid
    Pix q = (*conns)[cdfid]->Cdds().first_var();
    for (int i=0; i < varid ; i++)
	(*conns)[cdfid]->Cdds().next_var(q);

    if(name != NULL)
	(void) strcpy(name, (*conns)[cdfid]->Cdds().var(q)->name().c_str());

    return NC_NOERR;
}


int 
nc_inq_vartype(int cdfid, int varid, nc_type *typep)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode= lnc_inq_vartype((*conns)[cdfid]->ncid(), varid, typep);
      return rcode;
    }
#endif

    if(( varid < 0) || (varid > (*conns)[cdfid]->nvars()))
      return NC_ENOTVAR;

    //select the Pix for the given varid
    Pix q = (*conns)[cdfid]->Cdds().first_var();
    for (int i=0; i < varid ; i++)
	(*conns)[cdfid]->Cdds().next_var(q);

    BaseType *bt = (*conns)[cdfid]->Cdds().var(q);
    if(typep != 0)
	var_info(cdfid,bt,typep, 0, 0);

    return NC_NOERR; 
}


int 
nc_inq_varndims(int cdfid, int varid, int *ndimsp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

    if((*conns)[cdfid] == NULL) // does the file id exist?
      return NC_EBADID ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode= lnc_inq_varndims((*conns)[cdfid]->ncid(), varid, ndimsp);
      return rcode;
    }
#endif

    if(( varid < 0) || (varid > (*conns)[cdfid]->nvars()))
      return NC_ENOTVAR;

    //select the Pix for the given varid
    Pix q = (*conns)[cdfid]->Cdds().first_var();
    for (int i=0; i < varid ; i++)
	(*conns)[cdfid]->Cdds().next_var(q);

   BaseType *bt = (*conns)[cdfid]->Cdds().var(q);
   if(ndimsp != 0)
	var_info(cdfid, bt, 0, ndimsp, 0);

    return NC_NOERR; 
}


int 
nc_inq_vardimid(int cdfid, int varid, int *dimids)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode= lnc_inq_vardimid((*conns)[cdfid]->ncid(), varid, dimids);
      return rcode;
    }
#endif

    if(( varid < 0) || (varid > (*conns)[cdfid]->nvars()))
      return NC_ENOTVAR;

    //select the Pix for the given varid
    Pix q = (*conns)[cdfid]->Cdds().first_var();
    for (int i=0; i < varid ; i++)
	(*conns)[cdfid]->Cdds().next_var(q);

   BaseType *bt = (*conns)[cdfid]->Cdds().var(q);
   if(dimids != 0)
	var_info(cdfid, bt, 0, 0, dimids);

    return NC_NOERR;
}


int 
nc_inq_varnatts(int cdfid, int varid, int *nattsp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
      rcode= lnc_inq_varnatts((*conns)[cdfid]->ncid(), varid, nattsp);
      return rcode;
    }
#endif

    // varid = -1 is used for global atrributes and it is valid
    if(( varid+1 < 0) || (varid+1 > (*conns)[cdfid]->nvars()))
      return NC_ENOTVAR;

    //select the Pix for the given varid
    Pix q = (*conns)[cdfid]->Cdds().first_var();
    for (int i=0; i < varid ; i++)
	(*conns)[cdfid]->Cdds().next_var(q);

    if(nattsp != 0)
      *nattsp = num_attr(cdfid,varid); // Gets NC_GLOBAL when varid is -1
    
    return NC_NOERR;
}


int
ncvarinq(int cdfid, int varid, char *name, nc_type *typep,
	 int *ndimsp, int dims[], int *nattrsp)
{
    if (!conns || cdfid < 0 || cdfid > MAX_NC_OPEN 
	|| (*conns)[cdfid] == NULL) // does the file id exist?
	return NC_EBADID;

    NCConnect *c = (*conns)[cdfid];

#ifdef LOCAL
    if(c->is_local()) {
	rcode = lncvarinq(c->ncid(), varid, name, typep, ndimsp, dims, nattrsp);
	return rcode;
    }
#endif
    cdf_routine_name = "ncvarinq" ;

    // varid = -1 is used for global atrributes
    if(( varid < 0) || (varid > (*conns)[cdfid]->nvars()))
	{
	    NCadvise(NC_ENOTVAR, "%d is not a valid variable id", varid) ;
	    return(-1) ;
	}

    //select the Pix for the given varid
    DDS &cdds = c->Cdds() ;
    Pix q = cdds.first_var();
    for (int i=0; i < varid ; i++)
	cdds.next_var(q);

    BaseType *bt = cdds.var(q);
    if(name != NULL)
	(void) strcpy(name, bt->name().c_str());

    if((typep != 0)||(ndimsp != 0)||(dims != 0))
	var_info(cdfid,bt,typep,ndimsp,dims);

    if(nattrsp != 0)
	*nattrsp = num_attr(cdfid,varid);

    return (varid);
}

// this needs to move to the Dputget.cc file 
int
ncvarcopy(int incdf, int varid, int outcdf)
{
    static char name[MAX_NC_NAME];
    static char dname[MAX_NC_NAME];
    static int dims[MAX_VAR_DIMS];
    static int outdims[MAX_VAR_DIMS];
    static long cor[MAX_VAR_DIMS];
    static long edge[MAX_VAR_DIMS];
    int nattrs;
    int ndims;
    nc_type datatype;
    long size;
    long outsize;
    char *tmp_values;
    int i;

    // does the file id exist?
    if (!conns || incdf < 0 || incdf > MAX_NC_OPEN 
	|| outcdf < 0 || outcdf > MAX_NC_OPEN 
	|| (*conns)[incdf] == NULL || (*conns)[outcdf] == NULL) 
	return NC_EBADID;

// case 1, both input and output files are local

#ifdef LOCAL
    if((*conns)[incdf]->is_local() && (*conns)[outcdf]->is_local()) {
	rcode = lnc_copy_var((*conns)[incdf]->ncid(), varid, 
			  (*conns)[outcdf]->ncid());
	return rcode;
    }
#endif

    cdf_routine_name = "ncvarcopy" ;

// case 2, the output file is not local, Error writing over the net.

    if(!(*conns)[outcdf]->is_local()) {
	NCadvise(NC_EPERM, "DODS network interface is read-only") ;
	return(-1);
    }

// last case, input is from DODS file and output a local netcdf file

#ifdef LOCAL

    // varid = -1 is used for global atrributes
    if(( varid < 0) || (varid > (*conns)[incdf]->nvars()))
	{
	    NCadvise(NC_ENOTVAR, "%d is not a valid variable id", varid) ;
	    return(-1) ;
	}

    //get variable name and type, etc
    ncvarinq(incdf, varid, name, &datatype, &ndims, dims, &nattrs);

    //get all the dimensions and make the missing ones in the output file
    for(i=0; i<ndims; i++){

	(void) ncdiminq(incdf, dims[i], dname, &size);

	// is there such a dim. in the output file, if not make it
	int outdim_id = lncdimid((*conns)[outcdf]->ncid(), dname);
	if( outdim_id == -1){
	    lncredef((*conns)[outcdf]->ncid()); 
	    outdims[i] = lncdimdef((*conns)[outcdf]->ncid(), dname, size);
	    lncendef((*conns)[outcdf]->ncid());
	}
	else{
	    //is it the correct size?
	    (void) lncdiminq((*conns)[outcdf]->ncid(), outdim_id, (char *)0, &outsize);
	    if(size == outsize)
		outdims[i] = outdim_id;
	    else{
		NCadvise(NC_EBADDIM, "dimention's size miss-match") ;
		return(-1) ;
	    }
	}
    }


    //create the new variable
    lncredef((*conns)[outcdf]->ncid()); 
    int outvar = lncvardef((*conns)[outcdf]->ncid(),name,datatype,ndims,outdims);
    if(outvar == -1)
	return(-1);
    lncendef((*conns)[outcdf]->ncid());

    //loop all the attributes and copy them
    for(i=0; i<nattrs; i++){
	ncattname(incdf, varid, i, name);
	rcode=ncattcopy(incdf,varid,name,outcdf,outvar);
	if(rcode == -1)
	    return(-1);
    }

    //get variable shape and size
    long nels=1;
    for(i=0; i<ndims; i++){
	(void) ncdiminq(incdf, dims[i], (char *)0, &size);
	nels *= size;
	cor[i]=0;
	edge[i]= size;
    }

    tmp_values = new char [(nels*nctypelen(datatype))];

    //read the date values into memory
    (void) ncvarget(incdf, varid, cor, edge, (void *)tmp_values);

    //Write the data to local file
    rcode=lncvarput((*conns)[outcdf]->ncid(),outvar,cor,edge,(void *)tmp_values);

    delete [] tmp_values;
    return rcode;

#else

    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);

#endif /* Last case LOCAL */

    return(0) ;
}

int
nc_copy_var(int ncid_in, int varid, int ncid_out)
{

#ifdef LOCAL
    if((*conns)[ncid_in]->is_local() && (*conns)[ncid_out]->is_local()) {
	rcode = lnc_copy_var((*conns)[ncid_in]->ncid(), varid, 
			  (*conns)[ncid_out]->ncid());
	return rcode;
    }
#endif

    return ncvarcpy(ncid_in, varid, ncid_out);
  }

// $Log: Dvar.cc,v $
// Revision 1.5  2003/01/28 07:08:24  jimg
// Merged with release-3-2-8.
//
// Revision 1.4.4.1  2002/12/05 20:32:39  pwest
// Corrected problems with IteratorAdapter code and cleaned up file descriptors
// and memory management problems.
//
// Revision 1.4  2000/10/06 01:22:02  jimg
// Moved the CVS Log entries to the ends of files.
// Modified the read() methods to match the new definition in the dap library.
// Added exception handlers in various places to catch exceptions thrown
// by the dap library.
//
// Revision 1.3  1999/11/05 05:15:04  jimg
// Result of merge woth 3-1-0
//
// Revision 1.1.2.2  1999/10/29 05:05:20  jimg
// Reza's fixes plus the configure & Makefile update
//
// Revision 1.2  1999/10/21 13:19:06  reza
// IMAP and other bug fixed for version3.
//
// Revision 1.1.2.1  1999/10/15 19:50:56  jimg
// Changed return values and conditions for NC API entry points
//
// Revision 1.1  1999/07/28 00:22:42  jimg
// Added
//
// Revision 1.13.2.1  1999/05/27 17:43:23  reza
// Fixed bugs in string changes
//
// Revision 1.13  1999/05/07 23:45:31  jimg
// String --> string fixes
//
// Revision 1.12  1999/04/08 13:45:52  reza
// Added support for more data types.
//
// Revision 1.11  1999/03/30 05:20:55  reza
// Added support for the new data types (Int16, UInt16, and Float32).
//
// Revision 1.10  1996/09/17 17:06:12  jimg
// Merge the release-2-0 tagged files (which were off on a branch) back into
// the trunk revision.
//
// Revision 1.9.2.3  1996/09/17 00:26:13  jimg
// Merged changes from a side branch which contained various changes from
// Reza and Charles.
// Removed ncdump and netexec since ncdump is now in its own directory and
// netexec is no longer used.
//
// Revision 1.9.2.2  1996/07/10 21:43:51  jimg
// Changes for version 2.06. These fixed lingering problems from the migration
// from version 1.x to version 2.x.
// Removed some (but not all) warning generated with gcc's -Wall option.
//
// Revision 1.9.2.1  1996/06/25 22:04:14  jimg
// Version 2.0 from Reza.
//
// Revision 1.9  1995/11/18  11:33:22  reza
// Updated member function names for DAP-1.1.1.
//
// Revision 1.8  1995/07/09  21:33:37  jimg
// Added copyright notice.
//
// Revision 1.7  1995/06/29  15:42:05  jimg
// Fixed instances of delete which should have included brackets
//
// Revision 1.6  1995/06/28  20:22:38  jimg
// Replaced malloc calls with calls to new (and calls to free with calls to
// delete).
//
// Revision 1.5  1995/06/23  14:00:46  reza
// Added ncvarcpy to the library.
//
// Revision 1.4  1994/12/22  04:51:40  reza
// Updated to use DODS new named dimension capability.
//
// Revision 1.3  1994/12/08  19:01:31  reza
// Added support for local netCDF file access through the original netCDF code.
//
// Revision 1.2  1994/11/23  21:06:04  reza
// First working version.
//
// Revision 1.1  1994/11/03  04:35:48  reza
// Preliminary version of netCDF -> DODS library.
