// -*- C++ -*-

// (c) COPYRIGHT URI/MIT 1994-1996
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      reza            Reza Nekovei (reza@intcomm.net)

// Interface for subclass of Connect: subclassed to add ctor for netcdf.
//
// jhrg 9/29/94

#ifndef _ncconnect_h
#define _ncconnect_h 1

#ifndef __POWERPC__
#ifdef __GNUG__
#pragma interface
#endif
#endif

#include "AISConnect.h"		// Changed to AISConnect. 03/11/03 jhrg

class NCConnect: public AISConnect {
private:
    DAS _das;			// Added to compensate for changes in
				// Connect. 04/15/03 jhrg
    DDS _Cdds;                  // Current (Constrained) dds
    int _ncid;			// used only if LOCAL is true, otherwise == -1
    int _nvars;			// Number of variables in this file
    int _recdim;		// Dimension ID for unlimited dim.
    int _ndims;			// Number of dimensions found
    string _dim_name[MAX_NC_DIMS];
    int _dim_size[MAX_NC_DIMS];
    int _das_loc[MAX_NC_VARS];
    void init_list(int i);
    void parse_array_dims();
    void parse_grid_dims();
    void set_recdim();

public:
    NCConnect(const string &name);
    virtual ~NCConnect();

    int ncid();
    void set_ncid(int id);
    int ndims();
    int nvars();
    int recdim();
    int dim_size(const int dimid);
    const string &dim_name(const int dimid);
    int das_loc(const int varid);
    void parse_das_loc();
    void parse_dims();

    // I added these two mthods along with the field _das to compensate for
    // changes in Connect. 04/15/03 jhrg
    void request_das();
    DAS &das();

    void request_Cdds(const string &expr, const string &ext = "dds");
    DDS &Cdds();
};

/* 
 * $Log: NCConnect.h,v $
 * Revision 1.5.2.1  2003/06/24 11:36:32  rmorris
 * Removed #pragma interface directives for the OS X.
 *
 * Revision 1.5  2003/05/02 16:30:29  jimg
 * Fixes for the builds
 *
 * Revision 1.4  2003/01/28 07:08:24  jimg
 * Merged with release-3-2-8.
 *
 * Revision 1.3.4.2  2002/12/27 00:35:24  jimg
 * Added set_ncid(). See NCConnect for an explaination.
 *
 * Revision 1.3.4.1  2002/06/21 00:31:40  jimg
 * I changed many files throughout the source so that the 'make World' build
 * works with the new versions of Connect and libdap++ that use libcurl.
 * Most of these changes are either to Makefiles, configure scripts or to
 * the headers included by various C++ files. In a few places the oddities
 * of libwww forced us to hack up code and I've undone those and some of the
 * clients had code that supported libwww's generous tracing capabilities
 * (that's one part of libwww I'll miss); I had to remove support for that.
 * Once this code compiles and more work is done on Connect, I'll return to
 * each of these changes and polish them.
 *
 * Revision 1.3  2000/10/06 01:22:02  jimg
 * Moved the CVS Log entries to the ends of files.
 * Modified the read() methods to match the new definition in the dap library.
 * Added exception handlers in various places to catch exceptions thrown
 * by the dap library.
 *
 * Revision 1.2  1999/11/05 05:15:05  jimg
 * Result of merge woth 3-1-0
 *
 * Revision 1.1.2.1  1999/10/29 05:05:21  jimg
 * Reza's fixes plus the configure & Makefile update
 *
 * Revision 1.1  1999/07/28 00:22:42  jimg
 * Added
 *
 * Revision 1.10  1999/05/07 23:45:31  jimg
 * String --> string fixes
 *
 * Revision 1.9  1996/09/19 16:58:17  jimg
 * Made the NCConnect dtor virtual.
 *
 * Revision 1.8  1996/09/17 17:06:23  jimg
 * Merge the release-2-0 tagged files (which were off on a branch) back into
 * the trunk revision.
 *
 * Revision 1.6.4.2  1996/07/10 21:44:01  jimg
 * Changes for version 2.06. These fixed lingering problems from the migration
 * from version 1.x to version 2.x.
 * Removed some (but not all) warning generated with gcc's -Wall option.
 *
 * Revision 1.6.4.1  1996/06/25 22:04:24  jimg
 * Version 2.0 from Reza.
 *
 * Revision 1.6  1994/11/23  20:59:54  reza
 * Added heuristic dimension matching for Grid and Array classes.
 * Added Dimension handling routines (ndims(), dim_size(), dim_name(),
 * parse_dim(),and etc.)
 *
 * Revision 1.5  1994/11/18  21:33:32  reza
 * Added _nvars and nvars() for number of variables in the file.
 * Added _das_loc[] and parse_das_loc() to make a look-up table for variables
 * in dds vs. das.
 *
 * Revision 1.4  1994/11/03  05:45:08  reza
 * Moved error, request_das, and request _dds to the surrogate library for more
 * flexibility and simpler error handling.
 *
 * Revision 1.3  1994/10/28  19:09:04  jimg
 * Added _error, error() and fixed ncid() so that it retruns a reference.
 *
 * Revision 1.2  1994/10/05  20:23:31  jimg
 * Fixed errors in *.h files comments - CVS bites again.
 * Changed request_{das,dds} so that they use the field `_api_name'
 * instead of requiring callers to pass the api name.
 *
 * Revision 1.1  1994/10/05  18:02:15  jimg
 * First version of the connection management classes.
 * This commit also includes early versions of the test code.
 */

#endif

